@@ -16,10 +16,13 @@
 */
 
 #include "grn_cast.h"
+#include "grn_float.h"
 #include "grn_geo.h"
 #include "grn_pat.h"
 #include "grn_str.h"
 
+#include <groonga/bulk.hpp>
+
 #ifdef GRN_WITH_RAPIDJSON
 #  include "grn_db.h"
 #  include <groonga/smart_obj.hpp>
@@ -1556,58 +1559,183 @@ grn_caster_cast_bool(grn_ctx *ctx, grn_caster *caster)
   return rc;
 }
 
-#define NUM2DEST(getvalue, totext, tobool, totime, tofloat32, tofloat)         \
-  switch (caster->dest->header.domain) {                                       \
-  case GRN_DB_BOOL:                                                            \
-    tobool(ctx, caster->dest, getvalue(caster->src));                          \
-    break;                                                                     \
-  case GRN_DB_INT8:                                                            \
-    GRN_INT8_SET(ctx, caster->dest, getvalue(caster->src));                    \
-    break;                                                                     \
-  case GRN_DB_UINT8:                                                           \
-    GRN_UINT8_SET(ctx, caster->dest, getvalue(caster->src));                   \
-    break;                                                                     \
-  case GRN_DB_INT16:                                                           \
-    GRN_INT16_SET(ctx, caster->dest, getvalue(caster->src));                   \
-    break;                                                                     \
-  case GRN_DB_UINT16:                                                          \
-    GRN_UINT16_SET(ctx, caster->dest, getvalue(caster->src));                  \
-    break;                                                                     \
-  case GRN_DB_INT32:                                                           \
-    GRN_INT32_SET(ctx, caster->dest, getvalue(caster->src));                   \
-    break;                                                                     \
-  case GRN_DB_UINT32:                                                          \
-    GRN_UINT32_SET(ctx, caster->dest, getvalue(caster->src));                  \
-    break;                                                                     \
-  case GRN_DB_TIME:                                                            \
-    totime(ctx, caster->dest, getvalue(caster->src));                          \
-    break;                                                                     \
-  case GRN_DB_INT64:                                                           \
-    GRN_INT64_SET(ctx, caster->dest, getvalue(caster->src));                   \
-    break;                                                                     \
-  case GRN_DB_UINT64:                                                          \
-    GRN_UINT64_SET(ctx, caster->dest, getvalue(caster->src));                  \
-    break;                                                                     \
-  case GRN_DB_FLOAT32:                                                         \
-    tofloat32(ctx, caster->dest, getvalue(caster->src));                       \
-    break;                                                                     \
-  case GRN_DB_FLOAT:                                                           \
-    tofloat(ctx, caster->dest, getvalue(caster->src));                         \
-    break;                                                                     \
-  case GRN_DB_SHORT_TEXT:                                                      \
-  case GRN_DB_TEXT:                                                            \
-  case GRN_DB_LONG_TEXT:                                                       \
-    totext(ctx, caster->dest, getvalue(caster->src));                          \
-    break;                                                                     \
-  case GRN_DB_TOKYO_GEO_POINT:                                                 \
-  case GRN_DB_WGS84_GEO_POINT:                                                 \
-    rc = GRN_INVALID_ARGUMENT;                                                 \
-    break;                                                                     \
-  default:                                                                     \
-    rc = grn_caster_cast_to_record(ctx, caster);                               \
-    break;                                                                     \
+namespace {
+  template <typename NUMERIC>
+  grn_rc
+  num2time(grn_ctx *ctx, grn_caster *caster, NUMERIC value)
+  {
+    switch (caster->src->header.domain) {
+    case GRN_DB_TIME:
+    case GRN_DB_FLOAT32:
+    case GRN_DB_FLOAT:
+      return grn::bulk::set<NUMERIC>(ctx, caster->dest, value);
+    default:
+      return grn::bulk::set<int64_t>(ctx,
+                                     caster->dest,
+                                     static_cast<int64_t>(value) *
+                                       GRN_TIME_USEC_PER_SEC);
+    }
   }
 
+  template <typename FLOATING_POINT, typename NUMERIC>
+  std::enable_if_t<std::is_floating_point_v<FLOATING_POINT>, grn_rc>
+  num2float(grn_ctx *ctx, grn_caster *caster, NUMERIC value)
+  {
+    if (caster->src->header.domain == GRN_DB_TIME) {
+      return grn::bulk::set<FLOATING_POINT>(ctx,
+                                            caster->dest,
+                                            static_cast<FLOATING_POINT>(value) /
+                                              GRN_TIME_USEC_PER_SEC);
+    } else {
+      return grn::bulk::set<FLOATING_POINT>(ctx, caster->dest, value);
+    }
+  }
+
+#ifdef GRN_HAVE_BFLOAT16
+  template <typename FLOATING_POINT, typename NUMERIC>
+  std::enable_if_t<std::is_same_v<FLOATING_POINT, grn_bfloat16> &&
+                     std::is_same_v<NUMERIC, grn_bfloat16>,
+                   grn_rc>
+  num2float(grn_ctx *ctx, grn_caster *caster, NUMERIC value)
+  {
+    if (caster->src->header.domain == GRN_DB_TIME) {
+      return grn::bulk::set<FLOATING_POINT>(ctx,
+                                            caster->dest,
+                                            grn_bfloat32_to_float32(value) /
+                                              GRN_TIME_USEC_PER_SEC);
+    } else {
+      return grn::bulk::set<FLOATING_POINT>(ctx, caster->dest, value);
+    }
+  }
+
+  template <typename FLOATING_POINT, typename NUMERIC>
+  std::enable_if_t<std::is_same_v<FLOATING_POINT, grn_bfloat16> &&
+                     !std::is_same_v<NUMERIC, grn_bfloat16>,
+                   grn_rc>
+  num2float(grn_ctx *ctx, grn_caster *caster, NUMERIC value)
+  {
+    if (caster->src->header.domain == GRN_DB_TIME) {
+      return grn::bulk::set<FLOATING_POINT>(
+        ctx,
+        caster->dest,
+        grn::numeric::to_bfloat16(static_cast<double>(value) /
+                                  GRN_TIME_USEC_PER_SEC));
+    } else {
+      return grn::bulk::set<NUMERIC>(ctx, caster->dest, value);
+    }
+  }
+#endif
+
+  template <typename NUMERIC>
+  std::enable_if_t<std::is_integral_v<NUMERIC> && std::is_signed_v<NUMERIC>,
+                   grn_rc>
+  num2text(grn_ctx *ctx, grn_caster *caster, NUMERIC value)
+  {
+    return grn_text_lltoa(ctx, caster->dest, value);
+  }
+
+  template <typename NUMERIC>
+  std::enable_if_t<std::is_integral_v<NUMERIC> && !std::is_signed_v<NUMERIC>,
+                   grn_rc>
+  num2text(grn_ctx *ctx, grn_caster *caster, NUMERIC value)
+  {
+    return grn_text_ulltoa(ctx, caster->dest, value);
+  }
+
+  template <typename NUMERIC>
+  std::enable_if_t<std::is_same_v<NUMERIC, float>, grn_rc>
+  num2text(grn_ctx *ctx, grn_caster *caster, NUMERIC value)
+  {
+    return grn_text_f32toa(ctx, caster->dest, value);
+  }
+
+  template <typename NUMERIC>
+  std::enable_if_t<std::is_same_v<NUMERIC, double>, grn_rc>
+  num2text(grn_ctx *ctx, grn_caster *caster, NUMERIC value)
+  {
+    return grn_text_ftoa(ctx, caster->dest, value);
+  }
+
+  template <typename SOURCE>
+  grn_rc
+  num2dest(grn_ctx *ctx, grn_caster *caster)
+  {
+    switch (caster->dest->header.domain) {
+    case GRN_DB_BOOL:
+      return grn::bulk::set<SOURCE>(
+        ctx,
+        caster->dest,
+        grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_INT8:
+      return grn::bulk::set<int8_t>(
+        ctx,
+        caster->dest,
+        grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_UINT8:
+      return grn::bulk::set<uint8_t>(
+        ctx,
+        caster->dest,
+        grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_INT16:
+      return grn::bulk::set<int16_t>(
+        ctx,
+        caster->dest,
+        grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_UINT16:
+      return grn::bulk::set<uint16_t>(
+        ctx,
+        caster->dest,
+        grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_INT32:
+      return grn::bulk::set<int32_t>(
+        ctx,
+        caster->dest,
+        grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_UINT32:
+      return grn::bulk::set<uint32_t>(
+        ctx,
+        caster->dest,
+        grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_INT64:
+      return grn::bulk::set<int64_t>(
+        ctx,
+        caster->dest,
+        grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_UINT64:
+      return grn::bulk::set<uint64_t>(
+        ctx,
+        caster->dest,
+        grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_TIME:
+      return num2time(ctx, caster, grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+#ifdef GRN_HAVE_BFLOAT16
+    case GRN_DB_BFLOAT16:
+      return num2float<grn_bfloat16>(
+        ctx,
+        caster,
+        grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+#endif
+    case GRN_DB_FLOAT32:
+      return num2float<float>(ctx,
+                              caster,
+                              grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_FLOAT:
+      return num2float<double>(ctx,
+                               caster,
+                               grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_SHORT_TEXT:
+    case GRN_DB_TEXT:
+    case GRN_DB_LONG_TEXT:
+      return num2text(ctx, caster, grn::bulk::get<SOURCE>(ctx, caster->src, 0));
+    case GRN_DB_TOKYO_GEO_POINT:
+    case GRN_DB_WGS84_GEO_POINT:
+      return GRN_INVALID_ARGUMENT;
+    default:
+      return grn_caster_cast_to_record(ctx, caster);
+    }
+  }
+} // namespace
+
 #define TEXT2DEST(type, tonum, setvalue)                                       \
   do {                                                                         \
     const char *cur, *str = GRN_TEXT_VALUE(caster->src);                       \
@@ -1639,44 +1767,6 @@ grn_caster_cast_bool(grn_ctx *ctx, grn_caster *caster)
     }                                                                          \
   } while (0)
 
-#define NUM2BOOL(ctx, dest, value) GRN_BOOL_SET(ctx, dest, value != 0)
-#define FLOAT322BOOL(ctx, dest, value)                                         \
-  do {                                                                         \
-    float value_ = value;                                                      \
-    GRN_BOOL_SET(ctx, dest, value_ < -FLT_EPSILON || FLT_EPSILON < value_);    \
-  } while (0)
-#define FLOAT2BOOL(ctx, dest, value)                                           \
-  do {                                                                         \
-    double value_ = value;                                                     \
-    GRN_BOOL_SET(ctx, dest, value_ < -DBL_EPSILON || DBL_EPSILON < value_);    \
-  } while (0)
-
-#define NUM2TIME(ctx, dest, value)                                             \
-  GRN_TIME_SET(ctx, dest, (long long int)(value)*GRN_TIME_USEC_PER_SEC);
-#define TIME2TIME(ctx, dest, value) GRN_TIME_SET(ctx, dest, value);
-#define FLOAT322TIME(ctx, dest, value)                                         \
-  do {                                                                         \
-    int64_t usec = llroundf(value * GRN_TIME_USEC_PER_SEC);                    \
-    GRN_TIME_SET(ctx, dest, usec);                                             \
-  } while (0)
-#define FLOAT2TIME(ctx, dest, value)                                           \
-  do {                                                                         \
-    int64_t usec = llround(value * GRN_TIME_USEC_PER_SEC);                     \
-    GRN_TIME_SET(ctx, dest, usec);                                             \
-  } while (0)
-
-#define NUM2FLOAT(ctx, dest, value) GRN_FLOAT_SET(ctx, dest, value);
-#define TIME2FLOAT(ctx, dest, value)                                           \
-  GRN_FLOAT_SET(ctx, dest, (double)(value) / GRN_TIME_USEC_PER_SEC);
-#define FLOAT322FLOAT(ctx, dest, value) GRN_FLOAT_SET(ctx, dest, value);
-#define FLOAT2FLOAT(ctx, dest, value)   GRN_FLOAT_SET(ctx, dest, value);
-
-#define NUM2FLOAT32(ctx, dest, value)   GRN_FLOAT32_SET(ctx, dest, value);
-#define TIME2FLOAT32(ctx, dest, value)                                         \
-  GRN_FLOAT32_SET(ctx, dest, (float)(value) / GRN_TIME_USEC_PER_SEC);
-#define FLOAT322FLOAT32(ctx, dest, value) GRN_FLOAT32_SET(ctx, dest, value);
-#define FLOAT2FLOAT32(ctx, dest, value)   GRN_FLOAT32_SET(ctx, dest, value);
-
 static grn_rc
 grn_caster_cast_text_to_bulk(grn_ctx *ctx, grn_caster *caster)
 {
@@ -1737,6 +1827,9 @@ grn_caster_cast_text_to_bulk(grn_ctx *ctx, grn_caster *caster)
   case GRN_DB_UINT64:
     TEXT2DEST(int64_t, grn_atoll, GRN_UINT64_SET);
     break;
+#ifdef GRN_HAVE_BFLOAT16
+  case GRN_DB_BFLOAT16:
+#endif
   case GRN_DB_FLOAT32:
   case GRN_DB_FLOAT:
     {
@@ -1749,15 +1842,7 @@ grn_caster_cast_text_to_bulk(grn_ctx *ctx, grn_caster *caster)
                    GRN_TEXT_LEN(caster->src));
       GRN_TEXT_PUTC(ctx, &buf, '\0');
       errno = 0;
-      if (caster->dest->header.domain == GRN_DB_FLOAT32) {
-        float value;
-        value = strtof(GRN_TEXT_VALUE(&buf), &end);
-        if (!errno && end + 1 == GRN_BULK_CURR(&buf)) {
-          GRN_FLOAT32_SET(ctx, caster->dest, value);
-        } else {
-          rc = GRN_INVALID_ARGUMENT;
-        }
-      } else {
+      if (caster->dest->header.domain == GRN_DB_FLOAT) {
         double value;
         value = strtod(GRN_TEXT_VALUE(&buf), &end);
         if (!errno && end + 1 == GRN_BULK_CURR(&buf)) {
@@ -1765,6 +1850,20 @@ grn_caster_cast_text_to_bulk(grn_ctx *ctx, grn_caster *caster)
         } else {
           rc = GRN_INVALID_ARGUMENT;
         }
+      } else {
+        float value;
+        value = strtof(GRN_TEXT_VALUE(&buf), &end);
+        if (!errno && end + 1 == GRN_BULK_CURR(&buf)) {
+          if (caster->dest->header.domain == GRN_DB_FLOAT32) {
+            GRN_FLOAT32_SET(ctx, caster->dest, value);
+          } else {
+#ifdef GRN_HAVE_BFLOAT16
+            grn::bulk::set(ctx, caster->dest, value);
+#endif
+          }
+        } else {
+          rc = GRN_INVALID_ARGUMENT;
+        }
       }
       GRN_OBJ_FIN(ctx, &buf);
     }
@@ -2131,92 +2230,37 @@ grn_caster_cast(grn_ctx *ctx, grn_caster *caster)
     rc = grn_caster_cast_bool(ctx, caster);
     break;
   case GRN_DB_INT8:
-    NUM2DEST(GRN_INT8_VALUE,
-             grn_text_itoa,
-             NUM2BOOL,
-             NUM2TIME,
-             NUM2FLOAT32,
-             NUM2FLOAT);
+    rc = num2dest<int8_t>(ctx, caster);
     break;
   case GRN_DB_UINT8:
-    NUM2DEST(GRN_UINT8_VALUE,
-             grn_text_lltoa,
-             NUM2BOOL,
-             NUM2TIME,
-             NUM2FLOAT32,
-             NUM2FLOAT);
+    rc = num2dest<uint8_t>(ctx, caster);
     break;
   case GRN_DB_INT16:
-    NUM2DEST(GRN_INT16_VALUE,
-             grn_text_itoa,
-             NUM2BOOL,
-             NUM2TIME,
-             NUM2FLOAT32,
-             NUM2FLOAT);
+    rc = num2dest<int16_t>(ctx, caster);
     break;
   case GRN_DB_UINT16:
-    NUM2DEST(GRN_UINT16_VALUE,
-             grn_text_lltoa,
-             NUM2BOOL,
-             NUM2TIME,
-             NUM2FLOAT32,
-             NUM2FLOAT);
+    rc = num2dest<uint16_t>(ctx, caster);
     break;
   case GRN_DB_INT32:
-    NUM2DEST(GRN_INT32_VALUE,
-             grn_text_itoa,
-             NUM2BOOL,
-             NUM2TIME,
-             NUM2FLOAT32,
-             NUM2FLOAT);
+    rc = num2dest<int32_t>(ctx, caster);
     break;
   case GRN_DB_UINT32:
-    NUM2DEST(GRN_UINT32_VALUE,
-             grn_text_lltoa,
-             NUM2BOOL,
-             NUM2TIME,
-             NUM2FLOAT32,
-             NUM2FLOAT);
+    rc = num2dest<uint32_t>(ctx, caster);
     break;
   case GRN_DB_INT64:
-    NUM2DEST(GRN_INT64_VALUE,
-             grn_text_lltoa,
-             NUM2BOOL,
-             NUM2TIME,
-             NUM2FLOAT32,
-             NUM2FLOAT);
-    break;
-  case GRN_DB_TIME:
-    NUM2DEST(GRN_TIME_VALUE,
-             grn_text_lltoa,
-             NUM2BOOL,
-             TIME2TIME,
-             TIME2FLOAT32,
-             TIME2FLOAT);
+    rc = num2dest<int64_t>(ctx, caster);
     break;
   case GRN_DB_UINT64:
-    NUM2DEST(GRN_UINT64_VALUE,
-             grn_text_lltoa,
-             NUM2BOOL,
-             NUM2TIME,
-             NUM2FLOAT32,
-             NUM2FLOAT);
+    rc = num2dest<uint64_t>(ctx, caster);
     break;
   case GRN_DB_FLOAT32:
-    NUM2DEST(GRN_FLOAT32_VALUE,
-             grn_text_f32toa,
-             FLOAT322BOOL,
-             FLOAT322TIME,
-             FLOAT322FLOAT32,
-             FLOAT322FLOAT);
+    rc = num2dest<float>(ctx, caster);
     break;
   case GRN_DB_FLOAT:
-    NUM2DEST(GRN_FLOAT_VALUE,
-             grn_text_ftoa,
-             FLOAT2BOOL,
-             FLOAT2TIME,
-             FLOAT2FLOAT32,
-             FLOAT2FLOAT);
+    rc = num2dest<double>(ctx, caster);
+    break;
+  case GRN_DB_TIME:
+    rc = num2dest<int64_t>(ctx, caster);
     break;
   case GRN_DB_SHORT_TEXT:
   case GRN_DB_TEXT:
