@@ -20,38 +20,39 @@
 #include "grn.h"
 #include "grn_load.h"
 
-#ifdef  __cplusplus
+#ifdef __cplusplus
 extern "C" {
 #endif
 
 typedef struct _grn_arrow_stream_loader grn_arrow_stream_loader;
 
 grn_arrow_stream_loader *
-grn_arrow_stream_loader_open(grn_ctx *ctx,
-                             grn_loader *loader);
+grn_arrow_stream_loader_open(grn_ctx *ctx, grn_loader *loader);
 grn_rc
-grn_arrow_stream_loader_close(grn_ctx *ctx,
-                              grn_arrow_stream_loader *loader);
+grn_arrow_stream_loader_close(grn_ctx *ctx, grn_arrow_stream_loader *loader);
 grn_rc
 grn_arrow_stream_loader_consume(grn_ctx *ctx,
                                 grn_arrow_stream_loader *loader,
                                 const char *data,
                                 size_t data_size);
 
-
 typedef struct _grn_arrow_stream_writer grn_arrow_stream_writer;
 grn_arrow_stream_writer *
-grn_arrow_stream_writer_open(grn_ctx *ctx,
-                             grn_obj *output_buffer);
+grn_arrow_stream_writer_open(grn_ctx *ctx, grn_obj *output_buffer);
 grn_rc
-grn_arrow_stream_writer_close(grn_ctx *ctx,
-                              grn_arrow_stream_writer *writer);
+grn_arrow_stream_writer_close(grn_ctx *ctx, grn_arrow_stream_writer *writer);
 grn_rc
 grn_arrow_stream_writer_add_field(grn_ctx *ctx,
                                   grn_arrow_stream_writer *writer,
                                   const char *name,
                                   grn_obj *column);
 grn_rc
+grn_arrow_stream_writer_add_field_text_dictionary(
+  grn_ctx *ctx,
+  grn_arrow_stream_writer *writer,
+  const char *name,
+  grn_obj *index_type);
+grn_rc
 grn_arrow_stream_writer_add_field_union(grn_ctx *ctx,
                                         grn_arrow_stream_writer *writer,
                                         const char *name,
@@ -80,6 +81,12 @@ grn_arrow_stream_writer_add_column_text(grn_ctx *ctx,
                                         const char *value,
                                         size_t value_length);
 grn_rc
+grn_arrow_stream_writer_add_column_text_dictionary(
+  grn_ctx *ctx,
+  grn_arrow_stream_writer *writer,
+  const char *value,
+  size_t value_length);
+grn_rc
 grn_arrow_stream_writer_add_column_int8(grn_ctx *ctx,
                                         grn_arrow_stream_writer *writer,
                                         int8_t value);
@@ -103,6 +110,12 @@ grn_rc
 grn_arrow_stream_writer_add_column_uint64(grn_ctx *ctx,
                                           grn_arrow_stream_writer *writer,
                                           uint64_t value);
+#ifdef GRN_HAVE_BFLOAT16
+grn_rc
+grn_arrow_stream_writer_add_column_bfloat16(grn_ctx *ctx,
+                                            grn_arrow_stream_writer *writer,
+                                            grn_bfloat16 value);
+#endif
 grn_rc
 grn_arrow_stream_writer_add_column_float32(grn_ctx *ctx,
                                            grn_arrow_stream_writer *writer,
@@ -128,8 +141,7 @@ grn_arrow_stream_writer_add_column_union(grn_ctx *ctx,
                                          grn_arrow_stream_writer *writer,
                                          int8_t type);
 grn_rc
-grn_arrow_stream_writer_flush(grn_ctx *ctx,
-                              grn_arrow_stream_writer *writer);
+grn_arrow_stream_writer_flush(grn_ctx *ctx, grn_arrow_stream_writer *writer);
 
 #ifdef __cplusplus
 }
