@@ -22,10 +22,14 @@
 #include "grn_pat.h"
 #include "grn_str.h"
 
+#define GRN_HIGHLIGHTER_DEFAULT_OPEN_TAG  "<span class=\"keyword\">"
+#define GRN_HIGHLIGHTER_DEFAULT_CLOSE_TAG "</span>"
+
 typedef struct {
+  grn_id chunk_id;
   uint64_t offset;
   uint32_t length;
-  grn_bool have_overlap;
+  bool have_overlap;
   uint32_t first_character_length;
 } grn_highlighter_location;
 
@@ -42,24 +46,27 @@ grn_highlighter_location_compare(const void *data1, const void *data2)
   }
 }
 
-/*
- * TODO:
- *   * Support non HTML mode.
- *   * Support custom tag.
- */
 struct _grn_highlighter {
   grn_obj_header header;
 
-  grn_bool is_html_mode;
-  grn_bool need_prepared;
+  bool is_html_mode;
+  bool need_prepared;
   grn_obj raw_keywords;
 
+  bool is_sequential_class_tag_mode;
   struct {
-    const char *open;
-    size_t open_length;
-    const char *close;
-    size_t close_length;
-  } tag;
+    grn_obj open;
+    grn_obj close;
+  } sequential_class_tag;
+
+  struct {
+    grn_obj open;
+    grn_obj close;
+  } default_tag;
+
+  grn_obj open_tags;
+  grn_obj close_tags;
+  size_t n_tags;
 
   /* For lexicon mode */
   struct {
@@ -68,7 +75,6 @@ struct _grn_highlighter {
     grn_obj lazy_keywords;
     grn_obj lazy_keyword_ids;
     grn_obj *token_id_chunks;
-    grn_obj token_id_chunk_ids;
     grn_obj token_id_chunk;
     grn_obj token_ids;
     grn_obj token_locations;
@@ -78,7 +84,7 @@ struct _grn_highlighter {
   /* For patricia trie mode */
   struct {
     grn_obj *keywords;
-    grn_obj keyword_ids;
+    grn_obj normalizers;
   } pat;
 };
 
@@ -93,9 +99,7 @@ grn_highlighter_open(grn_ctx *ctx)
   if (!highlighter) {
     char errbuf[GRN_CTX_MSGSIZE];
     grn_strcpy(errbuf, GRN_CTX_MSGSIZE, ctx->errbuf);
-    ERR(ctx->rc,
-        "[highlighter][open] failed to allocate memory: %s",
-        errbuf);
+    ERR(ctx->rc, "[highlighter][open] failed to allocate memory: %s", errbuf);
     GRN_API_RETURN(NULL);
   }
 
@@ -104,14 +108,28 @@ grn_highlighter_open(grn_ctx *ctx)
   highlighter->header.flags = 0;
   highlighter->header.domain = GRN_ID_NIL;
 
-  highlighter->is_html_mode = GRN_TRUE;
-  highlighter->need_prepared = GRN_TRUE;
+  highlighter->is_html_mode = true;
+  highlighter->need_prepared = true;
   GRN_TEXT_INIT(&(highlighter->raw_keywords), GRN_OBJ_VECTOR);
 
-  highlighter->tag.open = "<span class=\"keyword\">";
-  highlighter->tag.open_length = strlen(highlighter->tag.open);
-  highlighter->tag.close = "</span>";
-  highlighter->tag.close_length = strlen(highlighter->tag.close);
+  highlighter->is_sequential_class_tag_mode = false;
+  GRN_TEXT_INIT(&(highlighter->sequential_class_tag.open), 0);
+  GRN_TEXT_INIT(&(highlighter->sequential_class_tag.close), 0);
+
+  GRN_TEXT_INIT(&(highlighter->default_tag.open), 0);
+  grn_highlighter_set_default_open_tag(ctx,
+                                       highlighter,
+                                       GRN_HIGHLIGHTER_DEFAULT_OPEN_TAG,
+                                       -1);
+  GRN_TEXT_INIT(&(highlighter->default_tag.close), 0);
+  grn_highlighter_set_default_close_tag(ctx,
+                                        highlighter,
+                                        GRN_HIGHLIGHTER_DEFAULT_CLOSE_TAG,
+                                        -1);
+
+  GRN_TEXT_INIT(&(highlighter->open_tags), GRN_OBJ_VECTOR);
+  GRN_TEXT_INIT(&(highlighter->close_tags), GRN_OBJ_VECTOR);
+  highlighter->n_tags = 0;
 
   highlighter->lexicon.object = NULL;
   highlighter->lexicon.encoding = GRN_ENC_NONE;
@@ -120,23 +138,22 @@ grn_highlighter_open(grn_ctx *ctx)
                   GRN_OBJ_VECTOR,
                   GRN_ID_NIL);
   highlighter->lexicon.token_id_chunks = NULL;
-  GRN_RECORD_INIT(&(highlighter->lexicon.token_id_chunk_ids),
+  GRN_TEXT_INIT(&(highlighter->lexicon.token_id_chunk), 0);
+  GRN_RECORD_INIT(&(highlighter->lexicon.token_ids),
                   GRN_OBJ_VECTOR,
                   GRN_ID_NIL);
-  GRN_TEXT_INIT(&(highlighter->lexicon.token_id_chunk), 0);
-  GRN_RECORD_INIT(&(highlighter->lexicon.token_ids), GRN_OBJ_VECTOR, GRN_ID_NIL);
   GRN_TEXT_INIT(&(highlighter->lexicon.token_locations), 0);
   GRN_TEXT_INIT(&(highlighter->lexicon.candidates), 0);
 
   highlighter->pat.keywords = NULL;
-  GRN_RECORD_INIT(&(highlighter->pat.keyword_ids), GRN_OBJ_VECTOR, GRN_ID_NIL);
+  GRN_TEXT_INIT(&(highlighter->pat.normalizers), 0);
+  grn_highlighter_set_normalizers(ctx, highlighter, "NormalizerAuto", -1);
 
   GRN_API_RETURN(highlighter);
 }
 
 grn_rc
-grn_highlighter_close(grn_ctx *ctx,
-                      grn_highlighter *highlighter)
+grn_highlighter_close(grn_ctx *ctx, grn_highlighter *highlighter)
 {
   GRN_API_ENTER;
 
@@ -144,7 +161,7 @@ grn_highlighter_close(grn_ctx *ctx,
     GRN_API_RETURN(ctx->rc);
   }
 
-  GRN_OBJ_FIN(ctx, &(highlighter->pat.keyword_ids));
+  GRN_OBJ_FIN(ctx, &(highlighter->pat.normalizers));
   if (highlighter->pat.keywords) {
     grn_obj_close(ctx, highlighter->pat.keywords);
   }
@@ -154,12 +171,20 @@ grn_highlighter_close(grn_ctx *ctx,
   if (highlighter->lexicon.token_id_chunks) {
     grn_obj_close(ctx, highlighter->lexicon.token_id_chunks);
   }
-  GRN_OBJ_FIN(ctx, &(highlighter->lexicon.token_id_chunk_ids));
   GRN_OBJ_FIN(ctx, &(highlighter->lexicon.token_id_chunk));
   GRN_OBJ_FIN(ctx, &(highlighter->lexicon.candidates));
   GRN_OBJ_FIN(ctx, &(highlighter->lexicon.token_locations));
   GRN_OBJ_FIN(ctx, &(highlighter->lexicon.token_ids));
 
+  GRN_OBJ_FIN(ctx, &(highlighter->open_tags));
+  GRN_OBJ_FIN(ctx, &(highlighter->close_tags));
+
+  GRN_OBJ_FIN(ctx, &(highlighter->default_tag.open));
+  GRN_OBJ_FIN(ctx, &(highlighter->default_tag.close));
+
+  GRN_OBJ_FIN(ctx, &(highlighter->sequential_class_tag.open));
+  GRN_OBJ_FIN(ctx, &(highlighter->sequential_class_tag.close));
+
   GRN_OBJ_FIN(ctx, &(highlighter->raw_keywords));
   GRN_FREE(highlighter);
 
@@ -167,64 +192,10 @@ grn_highlighter_close(grn_ctx *ctx,
 }
 
 static void
-grn_highlighter_remove_unused_ids(grn_ctx *ctx,
-                                  grn_obj *table,
-                                  grn_obj *added_ids,
-                                  const char *tag)
+grn_highlighter_prepare_lexicon(grn_ctx *ctx, grn_highlighter *highlighter)
 {
-  grn_table_cursor *cursor;
-  size_t n;
-  grn_id id;
-
-  cursor = grn_table_cursor_open(ctx,
-                                 table,
-                                 NULL, 0,
-                                 NULL, 0,
-                                 0, -1, 0);
-  if (!cursor) {
-    grn_rc rc = ctx->rc;
-    char errbuf[GRN_CTX_MSGSIZE];
-    if (rc == GRN_SUCCESS) {
-      rc = GRN_UNKNOWN_ERROR;
-    }
-    grn_strcpy(errbuf, GRN_CTX_MSGSIZE, ctx->errbuf);
-    ERR(rc,
-        "[highlighter][prepare]%s "
-        "failed to create a cursor for internal patricia trie: %s",
-        tag,
-        errbuf);
-    return;
-  }
-
-  n = GRN_BULK_VSIZE(added_ids) / sizeof(grn_id);
-  while ((id = grn_table_cursor_next(ctx, cursor)) != GRN_ID_NIL) {
-    size_t i;
-    grn_bool using = GRN_FALSE;
-
-    for (i = 0; i < n; i++) {
-      if (id == GRN_RECORD_VALUE_AT(added_ids, i)) {
-        using = GRN_TRUE;
-        break;
-      }
-    }
-
-    if (using) {
-      continue;
-    }
-
-    grn_table_cursor_delete(ctx, cursor);
-  }
-  grn_table_cursor_close(ctx, cursor);
-}
-
-static void
-grn_highlighter_prepare_lexicon(grn_ctx *ctx,
-                                grn_highlighter *highlighter)
-{
-  grn_bool have_token_id_chunks = GRN_FALSE;
   grn_obj *lazy_keywords = &(highlighter->lexicon.lazy_keywords);
   grn_id lexicon_id;
-  grn_obj *token_id_chunk_ids = &(highlighter->lexicon.token_id_chunk_ids);
   size_t i, n_keywords;
   grn_obj *token_id_chunk = &(highlighter->lexicon.token_id_chunk);
 
@@ -243,14 +214,12 @@ grn_highlighter_prepare_lexicon(grn_ctx *ctx,
                      NULL);
 
   if (highlighter->lexicon.token_id_chunks) {
-    /* TODO: It may be better that we remove all existing records here
-     * for many keywords case. */
-    have_token_id_chunks =
-      grn_table_size(ctx, highlighter->lexicon.token_id_chunks) > 0;
+    grn_table_truncate(ctx, highlighter->lexicon.token_id_chunks);
   } else {
     highlighter->lexicon.token_id_chunks =
       grn_table_create(ctx,
-                       NULL, 0,
+                       NULL,
+                       0,
                        NULL,
                        GRN_OBJ_TABLE_PAT_KEY,
                        grn_ctx_at(ctx, GRN_DB_SHORT_TEXT),
@@ -268,12 +237,8 @@ grn_highlighter_prepare_lexicon(grn_ctx *ctx,
           errbuf);
       return;
     }
-    token_id_chunk_ids->header.domain =
-      grn_obj_id(ctx, highlighter->lexicon.token_id_chunks);
   }
 
-  GRN_BULK_REWIND(token_id_chunk_ids);
-
   n_keywords = grn_vector_size(ctx, &(highlighter->raw_keywords));
   for (i = 0; i < n_keywords; i++) {
     const char *keyword;
@@ -331,53 +296,30 @@ grn_highlighter_prepare_lexicon(grn_ctx *ctx,
     grn_token_cursor_close(ctx, cursor);
 
     {
-      grn_id token_id_chunk_id;
-
-      {
-        grn_encoding encoding = ctx->encoding;
-        /* token_id_chunk is a binary data */
-        ctx->encoding = GRN_ENC_NONE;
-        token_id_chunk_id =
-          grn_table_add(ctx,
-                        highlighter->lexicon.token_id_chunks,
-                        GRN_TEXT_VALUE(token_id_chunk),
-                        (unsigned int)GRN_TEXT_LEN(token_id_chunk),
-                        NULL);
-        ctx->encoding = encoding;
-      }
-      if (!have_token_id_chunks) {
-        continue;
-      }
-      if (token_id_chunk_id == GRN_ID_NIL) {
-        continue;
-      }
-      GRN_RECORD_PUT(ctx, token_id_chunk_ids, token_id_chunk_id);
+      grn_encoding encoding = ctx->encoding;
+      /* token_id_chunk is a binary data */
+      ctx->encoding = GRN_ENC_NONE;
+      grn_table_add(ctx,
+                    highlighter->lexicon.token_id_chunks,
+                    GRN_TEXT_VALUE(token_id_chunk),
+                    (unsigned int)GRN_TEXT_LEN(token_id_chunk),
+                    NULL);
+      ctx->encoding = encoding;
     }
   }
-
-  if (have_token_id_chunks) {
-    grn_highlighter_remove_unused_ids(ctx,
-                                      highlighter->lexicon.token_id_chunks,
-                                      token_id_chunk_ids,
-                                      "[prepare][lexicon]");
-  }
 }
 
 static void
 grn_highlighter_prepare_patricia_trie(grn_ctx *ctx,
                                       grn_highlighter *highlighter)
 {
-  grn_bool have_keywords = GRN_FALSE;
-  grn_obj *keyword_ids = &(highlighter->pat.keyword_ids);
-
   if (highlighter->pat.keywords) {
-    /* TODO: It may be better that we remove all existing records here
-     * for many keywords case. */
-    have_keywords = grn_table_size(ctx, highlighter->pat.keywords) > 0;
+    grn_table_truncate(ctx, highlighter->pat.keywords);
   } else {
     highlighter->pat.keywords =
       grn_table_create(ctx,
-                       NULL, 0,
+                       NULL,
+                       0,
                        NULL,
                        GRN_OBJ_TABLE_PAT_KEY,
                        grn_ctx_at(ctx, GRN_DB_SHORT_TEXT),
@@ -395,17 +337,14 @@ grn_highlighter_prepare_patricia_trie(grn_ctx *ctx,
           errbuf);
       return;
     }
-    keyword_ids->header.domain = grn_obj_id(ctx, highlighter->pat.keywords);
   }
 
-  GRN_BULK_REWIND(keyword_ids);
-
   if (highlighter->lexicon.object) {
     grn_obj normalizers;
     GRN_TEXT_INIT(&normalizers, 0);
     grn_table_get_normalizers_string(ctx,
-                                    highlighter->lexicon.object,
-                                    &normalizers);
+                                     highlighter->lexicon.object,
+                                     &normalizers);
     grn_obj_set_info(ctx,
                      highlighter->pat.keywords,
                      GRN_INFO_NORMALIZERS,
@@ -414,8 +353,11 @@ grn_highlighter_prepare_patricia_trie(grn_ctx *ctx,
   } else {
     grn_obj_set_info(ctx,
                      highlighter->pat.keywords,
-                     GRN_INFO_NORMALIZER,
-                     grn_ctx_get(ctx, "NormalizerAuto", -1));
+                     GRN_INFO_NORMALIZERS,
+                     &(highlighter->pat.normalizers));
+  }
+  if (ctx->rc != GRN_SUCCESS) {
+    return;
   }
 
   {
@@ -425,7 +367,6 @@ grn_highlighter_prepare_patricia_trie(grn_ctx *ctx,
     for (i = 0; i < n; i++) {
       const char *keyword;
       unsigned int keyword_size;
-      grn_id id;
 
       keyword_size = grn_vector_get_element(ctx,
                                             &(highlighter->raw_keywords),
@@ -433,32 +374,17 @@ grn_highlighter_prepare_patricia_trie(grn_ctx *ctx,
                                             &keyword,
                                             NULL,
                                             NULL);
-      id = grn_table_add(ctx,
-                         highlighter->pat.keywords,
-                         keyword,
-                         keyword_size,
-                         NULL);
-      if (!have_keywords) {
-        continue;
-      }
-      if (id == GRN_ID_NIL) {
-        continue;
-      }
-      GRN_RECORD_PUT(ctx, keyword_ids, id);
+      grn_table_add(ctx,
+                    highlighter->pat.keywords,
+                    keyword,
+                    keyword_size,
+                    NULL);
     }
   }
-
-  if (have_keywords) {
-    grn_highlighter_remove_unused_ids(ctx,
-                                      highlighter->pat.keywords,
-                                      keyword_ids,
-                                      "[prepare][no-lexicon]");
-  }
 }
 
 static void
-grn_highlighter_prepare(grn_ctx *ctx,
-                        grn_highlighter *highlighter)
+grn_highlighter_prepare(grn_ctx *ctx, grn_highlighter *highlighter)
 {
   bool use_lexicon = false;
   if (highlighter->lexicon.object) {
@@ -481,7 +407,10 @@ grn_highlighter_prepare(grn_ctx *ctx,
   } else {
     grn_highlighter_prepare_patricia_trie(ctx, highlighter);
   }
-  highlighter->need_prepared = GRN_FALSE;
+  size_t n_open_tags = grn_vector_size(ctx, &(highlighter->open_tags));
+  size_t n_close_tags = grn_vector_size(ctx, &(highlighter->close_tags));
+  highlighter->n_tags = (n_open_tags == n_close_tags ? n_open_tags : 0);
+  highlighter->need_prepared = false;
 }
 
 static grn_bool
@@ -490,7 +419,7 @@ grn_ids_is_included(grn_id *ids, size_t n_ids, grn_id id)
   size_t i;
 
   for (i = 0; i < n_ids; i++) {
-    if (ids[i] == ids[0]) {
+    if (ids[i] == id) {
       return GRN_TRUE;
     }
   }
@@ -523,6 +452,65 @@ grn_highlighter_log_location(grn_ctx *ctx,
           text + location->offset);
 }
 
+static void
+grn_highlighter_highlight_get_ith_tag(grn_ctx *ctx,
+                                      grn_highlighter *highlighter,
+                                      size_t i,
+                                      const char **open_tag,
+                                      size_t *open_tag_length,
+                                      const char **close_tag,
+                                      size_t *close_tag_length)
+{
+  if (highlighter->is_sequential_class_tag_mode) {
+    size_t n_keywords = grn_vector_size(ctx, &(highlighter->raw_keywords));
+    i = i % n_keywords;
+    GRN_BULK_REWIND(&(highlighter->sequential_class_tag.open));
+    grn_text_printf(ctx,
+                    &(highlighter->sequential_class_tag.open),
+                    "<mark class=\"keyword-%" GRN_FMT_SIZE "\">",
+                    i);
+    GRN_TEXT_SETS(ctx, &(highlighter->sequential_class_tag.close), "</mark>");
+    *open_tag = GRN_TEXT_VALUE(&(highlighter->sequential_class_tag.open));
+    *open_tag_length = GRN_TEXT_LEN(&(highlighter->sequential_class_tag.open));
+    *close_tag = GRN_TEXT_VALUE(&(highlighter->sequential_class_tag.close));
+    *close_tag_length =
+      GRN_TEXT_LEN(&(highlighter->sequential_class_tag.close));
+  } else if (highlighter->n_tags == 0) {
+    *open_tag = GRN_TEXT_VALUE(&(highlighter->default_tag.open));
+    *open_tag_length = GRN_TEXT_LEN(&(highlighter->default_tag.open)) - 1;
+    *close_tag = GRN_TEXT_VALUE(&(highlighter->default_tag.close));
+    *close_tag_length = GRN_TEXT_LEN(&(highlighter->default_tag.close)) - 1;
+  } else {
+    i = i % highlighter->n_tags;
+    *open_tag_length = grn_vector_get_element(ctx,
+                                              &(highlighter->open_tags),
+                                              i,
+                                              open_tag,
+                                              NULL,
+                                              NULL);
+    *close_tag_length = grn_vector_get_element(ctx,
+                                               &(highlighter->close_tags),
+                                               i,
+                                               close_tag,
+                                               NULL,
+                                               NULL);
+  }
+}
+
+static void
+grn_highlighter_highlight_add_normal_text(grn_ctx *ctx,
+                                          grn_highlighter *highlighter,
+                                          grn_obj *output,
+                                          const char *text,
+                                          size_t text_length)
+{
+  if (highlighter->is_html_mode) {
+    grn_text_escape_xml(ctx, output, text, text_length);
+  } else {
+    GRN_TEXT_PUT(ctx, output, text, text_length);
+  }
+}
+
 static uint64_t
 grn_highlighter_highlight_lexicon_flush(grn_ctx *ctx,
                                         grn_log_level log_level,
@@ -534,25 +522,42 @@ grn_highlighter_highlight_lexicon_flush(grn_ctx *ctx,
                                         grn_highlighter_location *location,
                                         uint64_t offset)
 {
-  grn_highlighter_log_location(ctx, log_level, tag, location, text, text_length);
+  grn_highlighter_log_location(ctx,
+                               log_level,
+                               tag,
+                               location,
+                               text,
+                               text_length);
   if (location->offset > offset) {
-    grn_text_escape_xml(ctx,
-                        output,
-                        text + offset,
-                        (size_t)(location->offset - offset));
+    grn_highlighter_highlight_add_normal_text(
+      ctx,
+      highlighter,
+      output,
+      text + offset,
+      (size_t)(location->offset - offset));
   }
-  GRN_TEXT_PUT(ctx,
-               output,
-               highlighter->tag.open,
-               highlighter->tag.open_length);
-  grn_text_escape_xml(ctx,
-                      output,
-                      text + location->offset,
-                      location->length);
-  GRN_TEXT_PUT(ctx,
-               output,
-               highlighter->tag.close,
-               highlighter->tag.close_length);
+
+  const char *open_tag;
+  size_t open_tag_length;
+  const char *close_tag;
+  size_t close_tag_length;
+  grn_highlighter_highlight_get_ith_tag(ctx,
+                                        highlighter,
+                                        /* We need "- 1" here
+                                         * because grn_id doesn't
+                                         * use 0 (GRN_ID_NIL). */
+                                        location->chunk_id - 1,
+                                        &open_tag,
+                                        &open_tag_length,
+                                        &close_tag,
+                                        &close_tag_length);
+  GRN_TEXT_PUT(ctx, output, open_tag, open_tag_length);
+  grn_highlighter_highlight_add_normal_text(ctx,
+                                            highlighter,
+                                            output,
+                                            text + location->offset,
+                                            location->length);
+  GRN_TEXT_PUT(ctx, output, close_tag, close_tag_length);
   return location->offset + location->length;
 }
 
@@ -583,10 +588,7 @@ grn_highlighter_highlight_lexicon(grn_ctx *ctx,
   if (!cursor) {
     char errbuf[GRN_CTX_MSGSIZE];
     grn_strcpy(errbuf, GRN_CTX_MSGSIZE, ctx->errbuf);
-    ERR(ctx->rc,
-        "%s failed to start tokenizing: %s",
-        tag,
-        errbuf);
+    ERR(ctx->rc, "%s failed to start tokenizing: %s", tag, errbuf);
     return;
   }
 
@@ -646,8 +648,10 @@ grn_highlighter_highlight_lexicon(grn_ctx *ctx,
                                lexicon,
                                cursor,
                                id,
-                               keyword, keyword_length,
-                               GRN_CURSOR_PREFIX) {
+                               keyword,
+                               keyword_length,
+                               GRN_CURSOR_PREFIX)
+      {
         int added = 0;
 
         {
@@ -674,7 +678,8 @@ grn_highlighter_highlight_lexicon(grn_ctx *ctx,
         if (added) {
           GRN_RECORD_PUT(ctx, lazy_keyword_ids, id);
         }
-      } GRN_TABLE_EACH_END(ctx, cursor);
+      }
+      GRN_TABLE_EACH_END(ctx, cursor);
       GRN_LOG(ctx,
               log_level,
               "%s[prefix-search][end] %" GRN_FMT_SIZE ":<%.*s>",
@@ -703,11 +708,7 @@ grn_highlighter_highlight_lexicon(grn_ctx *ctx,
     for (i = 0; i < n_token_ids; i++) {
       grn_id chunk_id;
 
-      GRN_LOG(ctx,
-              log_level,
-              "%s[lcp-search][start] %" GRN_FMT_SIZE,
-              tag,
-              i);
+      GRN_LOG(ctx, log_level, "%s[lcp-search][start] %" GRN_FMT_SIZE, tag, i);
       {
         grn_encoding encoding = ctx->encoding;
         /* token_id_chunk is a binary data */
@@ -736,7 +737,8 @@ grn_highlighter_highlight_lexicon(grn_ctx *ctx,
         grn_highlighter_location candidate;
         grn_highlighter_location *first = raw_token_locations + i;
 
-        candidate.have_overlap = GRN_FALSE;
+        candidate.chunk_id = chunk_id;
+        candidate.have_overlap = false;
         candidate.first_character_length = 0;
 
         {
@@ -747,10 +749,9 @@ grn_highlighter_highlight_lexicon(grn_ctx *ctx,
         }
         n_ids = key_size / sizeof(grn_id);
         candidate.offset = first->offset;
-        if (n_ids == 1 &&
-            grn_ids_is_included(raw_lazy_keyword_ids,
-                                n_lazy_keyword_ids,
-                                ids[0])) {
+        if (n_ids == 1 && grn_ids_is_included(raw_lazy_keyword_ids,
+                                              n_lazy_keyword_ids,
+                                              ids[0])) {
           candidate.length = first->first_character_length;
         } else if (first->have_overlap && n_ids > 1) {
           candidate.length = first->first_character_length;
@@ -787,11 +788,7 @@ grn_highlighter_highlight_lexicon(grn_ctx *ctx,
                                      &candidate,
                                      text,
                                      text_length);
-        GRN_LOG(ctx,
-                log_level,
-                "%s[lcp-search][end] %" GRN_FMT_SIZE,
-                tag,
-                i);
+        GRN_LOG(ctx, log_level, "%s[lcp-search][end] %" GRN_FMT_SIZE, tag, i);
         i += n_ids - 1;
       }
     }
@@ -870,10 +867,12 @@ grn_highlighter_highlight_lexicon(grn_ctx *ctx,
                                                        previous,
                                                        offset);
       if (offset < text_length) {
-        grn_text_escape_xml(ctx,
-                            output,
-                            text + offset,
-                            (size_t)(text_length - offset));
+        grn_highlighter_highlight_add_normal_text(
+          ctx,
+          highlighter,
+          output,
+          text + offset,
+          (size_t)(text_length - offset));
       }
     }
     GRN_LOG(ctx,
@@ -910,33 +909,47 @@ grn_highlighter_highlight_patricia_trie(grn_ctx *ctx,
                           MAX_N_HITS,
                           &rest);
     for (i = 0; i < n_hits; i++) {
+      const char *open_tag;
+      size_t open_tag_length;
+      const char *close_tag;
+      size_t close_tag_length;
+      grn_highlighter_highlight_get_ith_tag(ctx,
+                                            highlighter,
+                                            /* We need "- 1" here
+                                             * because grn_id doesn't
+                                             * use 0 (GRN_ID_NIL). */
+                                            hits[i].id - 1,
+                                            &open_tag,
+                                            &open_tag_length,
+                                            &close_tag,
+                                            &close_tag_length);
+
       if ((hits[i].offset - previous_length) > 0) {
-        grn_text_escape_xml(ctx,
-                            output,
-                            current + previous_length,
-                            hits[i].offset - previous_length);
+        grn_highlighter_highlight_add_normal_text(ctx,
+                                                  highlighter,
+                                                  output,
+                                                  current + previous_length,
+                                                  hits[i].offset -
+                                                    previous_length);
       }
-      GRN_TEXT_PUT(ctx,
-                   output,
-                   highlighter->tag.open,
-                   highlighter->tag.open_length);
-      grn_text_escape_xml(ctx,
-                          output,
-                          current + hits[i].offset,
-                          hits[i].length);
-      GRN_TEXT_PUT(ctx,
-                   output,
-                   highlighter->tag.close,
-                   highlighter->tag.close_length);
+      GRN_TEXT_PUT(ctx, output, open_tag, open_tag_length);
+      grn_highlighter_highlight_add_normal_text(ctx,
+                                                highlighter,
+                                                output,
+                                                current + hits[i].offset,
+                                                hits[i].length);
+      GRN_TEXT_PUT(ctx, output, close_tag, close_tag_length);
       previous_length = hits[i].offset + hits[i].length;
     }
 
     chunk_length = (size_t)(rest - current);
     if ((chunk_length - previous_length) > 0) {
-      grn_text_escape_xml(ctx,
-                          output,
-                          current + previous_length,
-                          current_length - previous_length);
+      grn_highlighter_highlight_add_normal_text(ctx,
+                                                highlighter,
+                                                output,
+                                                current + previous_length,
+                                                current_length -
+                                                  previous_length);
     }
     current_length -= chunk_length;
     current = rest;
@@ -957,15 +970,16 @@ grn_highlighter_highlight(grn_ctx *ctx,
     text_length = (int64_t)strlen(text);
   }
 
+  if (text_length == 0) {
+    goto exit;
+  }
+
   if (grn_vector_size(ctx, &(highlighter->raw_keywords)) == 0) {
-    if (highlighter->is_html_mode) {
-      grn_text_escape_xml(ctx,
-                          output,
-                          text,
-                          (size_t)text_length);
-    } else {
-      GRN_TEXT_PUT(ctx, output, text, text_length);
-    }
+    grn_highlighter_highlight_add_normal_text(ctx,
+                                              highlighter,
+                                              output,
+                                              text,
+                                              (size_t)text_length);
     goto exit;
   }
 
@@ -990,7 +1004,7 @@ grn_highlighter_highlight(grn_ctx *ctx,
                                       output);
   }
 
-exit :
+exit:
   GRN_API_RETURN(ctx->rc);
 }
 
@@ -1002,7 +1016,7 @@ grn_highlighter_set_lexicon(grn_ctx *ctx,
   GRN_API_ENTER;
 
   if (highlighter->lexicon.object != lexicon) {
-    highlighter->need_prepared = GRN_TRUE;
+    highlighter->need_prepared = true;
     highlighter->lexicon.object = lexicon;
   }
 
@@ -1010,14 +1024,113 @@ grn_highlighter_set_lexicon(grn_ctx *ctx,
 }
 
 grn_obj *
-grn_highlighter_get_lexicon(grn_ctx *ctx,
-                            grn_highlighter *highlighter)
+grn_highlighter_get_lexicon(grn_ctx *ctx, grn_highlighter *highlighter)
 {
   GRN_API_ENTER;
 
   GRN_API_RETURN(highlighter->lexicon.object);
 }
 
+grn_rc
+grn_highlighter_set_default_open_tag(grn_ctx *ctx,
+                                     grn_highlighter *highlighter,
+                                     const char *tag,
+                                     int64_t tag_length)
+{
+  GRN_API_ENTER;
+  if (tag_length < 0) {
+    tag_length = strlen(tag);
+  }
+  GRN_TEXT_SET(ctx, &(highlighter->default_tag.open), tag, tag_length);
+  GRN_TEXT_PUTC(ctx, &(highlighter->default_tag.open), '\0');
+  GRN_API_RETURN(ctx->rc);
+}
+
+const char *
+grn_highlighter_get_default_open_tag(grn_ctx *ctx, grn_highlighter *highlighter)
+{
+  return GRN_TEXT_VALUE(&(highlighter->default_tag.open));
+}
+
+grn_rc
+grn_highlighter_set_default_close_tag(grn_ctx *ctx,
+                                      grn_highlighter *highlighter,
+                                      const char *tag,
+                                      int64_t tag_length)
+{
+  GRN_API_ENTER;
+  if (tag_length < 0) {
+    tag_length = strlen(tag);
+  }
+  GRN_TEXT_SET(ctx, &(highlighter->default_tag.close), tag, tag_length);
+  GRN_TEXT_PUTC(ctx, &(highlighter->default_tag.close), '\0');
+  GRN_API_RETURN(ctx->rc);
+}
+
+const char *
+grn_highlighter_get_default_close_tag(grn_ctx *ctx,
+                                      grn_highlighter *highlighter)
+{
+  return GRN_TEXT_VALUE(&(highlighter->default_tag.close));
+}
+
+grn_rc
+grn_highlighter_set_sequential_class_tag_mode(grn_ctx *ctx,
+                                              grn_highlighter *highlighter,
+                                              bool mode)
+{
+  GRN_API_ENTER;
+  highlighter->is_sequential_class_tag_mode = mode;
+  GRN_API_RETURN(ctx->rc);
+}
+
+bool
+grn_highlighter_get_sequential_class_tag_mode(grn_ctx *ctx,
+                                              grn_highlighter *highlighter)
+{
+  return highlighter->is_sequential_class_tag_mode;
+}
+
+grn_rc
+grn_highlighter_set_normalizers(grn_ctx *ctx,
+                                grn_highlighter *highlighter,
+                                const char *normalizers,
+                                int64_t normalizers_length)
+{
+  GRN_API_ENTER;
+  if (normalizers_length < 0) {
+    normalizers_length = strlen(normalizers);
+  }
+  GRN_TEXT_SET(ctx,
+               &(highlighter->pat.normalizers),
+               normalizers,
+               normalizers_length);
+  highlighter->need_prepared = true;
+  GRN_API_RETURN(ctx->rc);
+}
+
+grn_obj *
+grn_highlighter_get_normalizers(grn_ctx *ctx, grn_highlighter *highlighter)
+{
+  return &(highlighter->pat.normalizers);
+}
+
+grn_rc
+grn_highlighter_set_html_mode(grn_ctx *ctx,
+                              grn_highlighter *highlighter,
+                              bool html_mode)
+{
+  GRN_API_ENTER;
+  highlighter->is_html_mode = html_mode;
+  GRN_API_RETURN(ctx->rc);
+}
+
+bool
+grn_highlighter_get_html_mode(grn_ctx *ctx, grn_highlighter *highlighter)
+{
+  return highlighter->is_html_mode;
+}
+
 grn_rc
 grn_highlighter_add_keyword(grn_ctx *ctx,
                             grn_highlighter *highlighter,
@@ -1060,22 +1173,71 @@ grn_highlighter_add_keyword(grn_ctx *ctx,
                          (uint32_t)keyword_length,
                          0,
                          GRN_DB_TEXT);
-  highlighter->need_prepared = GRN_TRUE;
+  highlighter->need_prepared = true;
 
-exit :
+exit:
   GRN_API_RETURN(ctx->rc);
 }
 
 grn_rc
-grn_highlighter_clear_keywords(grn_ctx *ctx,
-                               grn_highlighter *highlighter)
+grn_highlighter_clear_keywords(grn_ctx *ctx, grn_highlighter *highlighter)
 {
   grn_obj *raw_keywords = &(highlighter->raw_keywords);
 
   GRN_API_ENTER;
 
   GRN_BULK_REWIND(raw_keywords);
-  highlighter->need_prepared = GRN_TRUE;
+  highlighter->need_prepared = true;
 
   GRN_API_RETURN(ctx->rc);
 }
+
+grn_rc
+grn_highlighter_add_open_tag(grn_ctx *ctx,
+                             grn_highlighter *highlighter,
+                             const char *tag,
+                             int64_t tag_length)
+{
+  GRN_API_ENTER;
+  if (tag_length < 0) {
+    tag_length = strlen(tag);
+  }
+  grn_vector_add_element(ctx,
+                         &(highlighter->open_tags),
+                         tag,
+                         tag_length,
+                         0,
+                         GRN_DB_TEXT);
+  highlighter->need_prepared = true;
+  GRN_API_RETURN(ctx->rc);
+}
+
+grn_rc
+grn_highlighter_add_close_tag(grn_ctx *ctx,
+                              grn_highlighter *highlighter,
+                              const char *tag,
+                              int64_t tag_length)
+{
+  GRN_API_ENTER;
+  if (tag_length < 0) {
+    tag_length = strlen(tag);
+  }
+  grn_vector_add_element(ctx,
+                         &(highlighter->close_tags),
+                         tag,
+                         tag_length,
+                         0,
+                         GRN_DB_TEXT);
+  highlighter->need_prepared = true;
+  GRN_API_RETURN(ctx->rc);
+}
+
+grn_rc
+grn_highlighter_clear_tags(grn_ctx *ctx, grn_highlighter *highlighter)
+{
+  GRN_API_ENTER;
+  GRN_BULK_REWIND(&(highlighter->open_tags));
+  GRN_BULK_REWIND(&(highlighter->close_tags));
+  highlighter->need_prepared = true;
+  GRN_API_RETURN(ctx->rc);
+}
