@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2015-2018  Brazil
-  Copyright (C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018-2023  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -36,22 +36,22 @@ grn_obj_set_flag_to_string(int flags)
   const char *message = "invalid set flag";
 
   switch (flags & GRN_OBJ_SET_MASK) {
-  case GRN_OBJ_SET :
+  case GRN_OBJ_SET:
     message = "set";
     break;
-  case GRN_OBJ_INCR :
+  case GRN_OBJ_INCR:
     message = "increment";
     break;
-  case GRN_OBJ_DECR :
+  case GRN_OBJ_DECR:
     message = "decrement";
     break;
-  case GRN_OBJ_APPEND :
+  case GRN_OBJ_APPEND:
     message = "append";
     break;
-  case GRN_OBJ_PREPEND :
+  case GRN_OBJ_PREPEND:
     message = "prepend";
     break;
-  default :
+  default:
     break;
   }
 
@@ -66,29 +66,39 @@ grn_obj_is_true(grn_ctx *ctx, grn_obj *obj)
   }
 
   switch (obj->header.type) {
-  case GRN_BULK :
+  case GRN_BULK:
     switch (obj->header.domain) {
-    case GRN_DB_BOOL :
+    case GRN_DB_BOOL:
       return GRN_BOOL_VALUE(obj);
-    case GRN_DB_INT32 :
+    case GRN_DB_INT32:
       return GRN_INT32_VALUE(obj) != 0;
-    case GRN_DB_UINT32 :
+    case GRN_DB_UINT32:
       return GRN_UINT32_VALUE(obj) != 0;
-    case GRN_DB_FLOAT32 : {
-      float float_value;
-      float_value = GRN_FLOAT32_VALUE(obj);
-      return grn_float32_is_zero(float_value);
-    }
-    case GRN_DB_FLOAT : {
-      double float_value;
-      float_value = GRN_FLOAT_VALUE(obj);
-      return grn_float_is_zero(float_value);
-    }
-    case GRN_DB_SHORT_TEXT :
-    case GRN_DB_TEXT :
-    case GRN_DB_LONG_TEXT :
+#ifdef GRN_HAVE_BFLOAT16
+    case GRN_DB_BFLOAT16:
+      {
+        grn_bfloat16 bfloat16_value;
+        bfloat16_value = GRN_BFLOAT16_VALUE(obj);
+        return grn_bfloat16_is_zero(bfloat16_value);
+      }
+#endif
+    case GRN_DB_FLOAT32:
+      {
+        float float_value;
+        float_value = GRN_FLOAT32_VALUE(obj);
+        return grn_float32_is_zero(float_value);
+      }
+    case GRN_DB_FLOAT:
+      {
+        double float_value;
+        float_value = GRN_FLOAT_VALUE(obj);
+        return grn_float_is_zero(float_value);
+      }
+    case GRN_DB_SHORT_TEXT:
+    case GRN_DB_TEXT:
+    case GRN_DB_LONG_TEXT:
       return GRN_TEXT_LEN(obj) != 0;
-    default :
+    default:
       if (grn_id_maybe_table(ctx, obj->header.domain)) {
         return GRN_BULK_VSIZE(obj) > 0 && GRN_PTR_VALUE(obj) != GRN_ID_NIL;
       } else {
@@ -96,19 +106,19 @@ grn_obj_is_true(grn_ctx *ctx, grn_obj *obj)
       }
     }
     break;
-  case GRN_VECTOR :
+  case GRN_VECTOR:
     if (grn_ctx_get_command_version(ctx) < GRN_COMMAND_VERSION_3) {
       return true;
     } else {
       return grn_vector_size(ctx, obj) > 0;
     }
-  case GRN_UVECTOR :
+  case GRN_UVECTOR:
     if (grn_ctx_get_command_version(ctx) < GRN_COMMAND_VERSION_3) {
       return true;
     } else {
       return grn_uvector_size(ctx, obj) > 0;
     }
-  default :
+  default:
     return false;
   }
 }
@@ -140,7 +150,9 @@ grn_obj_is_builtin(grn_ctx *ctx, grn_obj *obj)
 {
   grn_id id;
 
-  if (!obj) { return GRN_FALSE; }
+  if (!obj) {
+    return GRN_FALSE;
+  }
 
   id = grn_obj_id(ctx, obj);
   return grn_id_is_builtin(ctx, id);
@@ -246,13 +258,13 @@ grn_obj_is_table(grn_ctx *ctx, grn_obj *obj)
   }
 
   switch (obj->header.type) {
-  case GRN_TABLE_NO_KEY :
-  case GRN_TABLE_HASH_KEY :
-  case GRN_TABLE_PAT_KEY :
-  case GRN_TABLE_DAT_KEY :
+  case GRN_TABLE_NO_KEY:
+  case GRN_TABLE_HASH_KEY:
+  case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
     is_table = GRN_TRUE;
     break;
-  default :
+  default:
     break;
   }
 
@@ -269,11 +281,11 @@ grn_obj_is_table_with_key(grn_ctx *ctx, grn_obj *obj)
   }
 
   switch (obj->header.type) {
-  case GRN_TABLE_HASH_KEY :
-  case GRN_TABLE_PAT_KEY :
-  case GRN_TABLE_DAT_KEY :
+  case GRN_TABLE_HASH_KEY:
+  case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
     is_table_with_key = true;
-  default :
+  default:
     break;
   }
 
@@ -290,12 +302,12 @@ grn_obj_is_table_with_value(grn_ctx *ctx, grn_obj *obj)
   }
 
   switch (obj->header.type) {
-  case GRN_TABLE_HASH_KEY :
-  case GRN_TABLE_PAT_KEY :
-  case GRN_TABLE_DAT_KEY :
-  case GRN_TABLE_NO_KEY :
+  case GRN_TABLE_HASH_KEY:
+  case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
+  case GRN_TABLE_NO_KEY:
     is_table_with_value = DB_OBJ(obj)->range != GRN_ID_NIL;
-  default :
+  default:
     break;
   }
 
@@ -320,7 +332,8 @@ grn_obj_is_lexicon(grn_ctx *ctx, grn_obj *obj)
   grn_table_columns(ctx, obj, "", 0, (grn_obj *)columns);
   bool is_lexicon = false;
   if (grn_hash_size(ctx, columns) > 0) {
-    GRN_HASH_EACH_BEGIN(ctx, columns, cursor, id) {
+    GRN_HASH_EACH_BEGIN(ctx, columns, cursor, id)
+    {
       void *key;
       grn_hash_cursor_get_key(ctx, cursor, &key);
       grn_id column_id = *((grn_id *)key);
@@ -332,7 +345,8 @@ grn_obj_is_lexicon(grn_ctx *ctx, grn_obj *obj)
       if (is_lexicon) {
         break;
       }
-    } GRN_HASH_EACH_END(ctx, cursor);
+    }
+    GRN_HASH_EACH_END(ctx, cursor);
   }
   grn_hash_close(ctx, columns);
   return is_lexicon;
@@ -359,7 +373,8 @@ grn_obj_is_lexicon_without_data_column(grn_ctx *ctx, grn_obj *obj)
     is_lexicon_without_data_column = false;
   } else {
     is_lexicon_without_data_column = true;
-    GRN_HASH_EACH_BEGIN(ctx, columns, cursor, id) {
+    GRN_HASH_EACH_BEGIN(ctx, columns, cursor, id)
+    {
       void *key;
       grn_hash_cursor_get_key(ctx, cursor, &key);
       grn_id column_id = *((grn_id *)key);
@@ -371,7 +386,8 @@ grn_obj_is_lexicon_without_data_column(grn_ctx *ctx, grn_obj *obj)
       if (!is_lexicon_without_data_column) {
         break;
       }
-    } GRN_HASH_EACH_END(ctx, cursor);
+    }
+    GRN_HASH_EACH_END(ctx, cursor);
   }
   grn_hash_close(ctx, columns);
   return is_lexicon_without_data_column;
@@ -427,11 +443,11 @@ grn_obj_is_column(grn_ctx *ctx, grn_obj *obj)
   }
 
   switch (obj->header.type) {
-  case GRN_COLUMN_FIX_SIZE :
-  case GRN_COLUMN_VAR_SIZE :
-  case GRN_COLUMN_INDEX :
+  case GRN_COLUMN_FIX_SIZE:
+  case GRN_COLUMN_VAR_SIZE:
+  case GRN_COLUMN_INDEX:
     is_column = GRN_TRUE;
-  default :
+  default:
     break;
   }
 
@@ -455,7 +471,8 @@ grn_obj_is_scalar_column(grn_ctx *ctx, grn_obj *obj)
     return GRN_FALSE;
   }
 
-  return (obj->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) == GRN_OBJ_COLUMN_SCALAR;
+  return (obj->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) ==
+         GRN_OBJ_COLUMN_SCALAR;
 }
 
 bool
@@ -485,9 +502,9 @@ grn_obj_is_vector_column(grn_ctx *ctx, grn_obj *obj)
     return GRN_FALSE;
   }
 
-  return ((obj->header.type == GRN_COLUMN_VAR_SIZE) &&
-          ((obj->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) ==
-           GRN_OBJ_COLUMN_VECTOR));
+  return (
+    (obj->header.type == GRN_COLUMN_VAR_SIZE) &&
+    ((obj->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) == GRN_OBJ_COLUMN_VECTOR));
 }
 
 bool
@@ -549,7 +566,7 @@ grn_obj_is_data_column(grn_ctx *ctx, grn_obj *obj)
   }
 
   return obj->header.type == GRN_COLUMN_FIX_SIZE ||
-    obj->header.type == GRN_COLUMN_VAR_SIZE;
+         obj->header.type == GRN_COLUMN_VAR_SIZE;
 }
 
 grn_bool
@@ -784,6 +801,40 @@ grn_obj_is_column_value_accessor(grn_ctx *ctx, grn_obj *obj)
   return accessor->action == GRN_ACCESSOR_GET_COLUMN_VALUE;
 }
 
+bool
+grn_obj_is_number_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_accessor *accessor;
+
+  if (!grn_obj_is_accessor(ctx, obj)) {
+    return false;
+  }
+
+  accessor = (grn_accessor *)obj;
+  while (accessor->next) {
+    accessor = accessor->next;
+  }
+
+  switch (accessor->action) {
+  case GRN_ACCESSOR_GET_ID:
+  case GRN_ACCESSOR_GET_SCORE:
+  case GRN_ACCESSOR_GET_NSUBRECS:
+  case GRN_ACCESSOR_GET_MAX:
+  case GRN_ACCESSOR_GET_MIN:
+  case GRN_ACCESSOR_GET_SUM:
+  case GRN_ACCESSOR_GET_AVG:
+  case GRN_ACCESSOR_GET_MEAN:
+    return true;
+  case GRN_ACCESSOR_GET_VALUE:
+    return grn_type_id_is_number_family(ctx,
+                                        grn_obj_get_range(ctx, accessor->obj));
+  case GRN_ACCESSOR_GET_COLUMN_VALUE:
+    return grn_obj_is_number_family_scalar_column(ctx, accessor->obj);
+  default:
+    return false;
+  }
+}
+
 grn_bool
 grn_obj_is_type(grn_ctx *ctx, grn_obj *obj)
 {
@@ -950,28 +1001,28 @@ grn_obj_is_visible(grn_ctx *ctx, grn_obj *obj)
 
   uint32_t flags = 0;
   switch (obj->header.type) {
-  case GRN_TABLE_HASH_KEY :
+  case GRN_TABLE_HASH_KEY:
     flags = ((grn_hash *)obj)->header.common->flags;
     break;
-  case GRN_TABLE_PAT_KEY :
+  case GRN_TABLE_PAT_KEY:
     flags = ((grn_pat *)obj)->header->flags;
     break;
-  case GRN_TABLE_DAT_KEY :
+  case GRN_TABLE_DAT_KEY:
     flags = ((grn_dat *)obj)->header->flags;
     break;
-  case GRN_TABLE_NO_KEY :
+  case GRN_TABLE_NO_KEY:
     flags = grn_array_get_flags(ctx, ((grn_array *)obj));
     break;
-  case GRN_COLUMN_FIX_SIZE :
+  case GRN_COLUMN_FIX_SIZE:
     flags = ((grn_ra *)obj)->header->flags;
     break;
-  case GRN_COLUMN_VAR_SIZE :
+  case GRN_COLUMN_VAR_SIZE:
     flags = grn_ja_get_flags(ctx, (grn_ja *)obj);
     break;
-  case GRN_COLUMN_INDEX :
+  case GRN_COLUMN_INDEX:
     flags = ((grn_ii *)obj)->header.common->flags;
     break;
-  default :
+  default:
     return false;
     break;
   }
@@ -980,15 +1031,12 @@ grn_obj_is_visible(grn_ctx *ctx, grn_obj *obj)
 }
 
 grn_rc
-grn_obj_set_visibility(grn_ctx *ctx,
-                       grn_obj *obj,
-                       bool is_visible)
+grn_obj_set_visibility(grn_ctx *ctx, grn_obj *obj, bool is_visible)
 {
   GRN_API_ENTER;
 
   if (!obj) {
-    ERR(GRN_INVALID_ARGUMENT,
-        "[obj][set-visibility] must not be NULL");
+    ERR(GRN_INVALID_ARGUMENT, "[obj][set-visibility] must not be NULL");
     GRN_API_RETURN(ctx->rc);
   }
 
@@ -1016,8 +1064,7 @@ grn_obj_set_visibility(grn_ctx *ctx,
 }
 
 bool
-grn_obj_have_source(grn_ctx *ctx,
-                    grn_obj *obj)
+grn_obj_have_source(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
     return false;
@@ -1042,10 +1089,8 @@ grn_db_reindex(grn_ctx *ctx, grn_obj *db)
   grn_table_cursor *cursor;
   grn_id id;
 
-  cursor = grn_table_cursor_open(ctx, db,
-                                 NULL, 0, NULL, 0,
-                                 0, -1,
-                                 GRN_CURSOR_BY_ID);
+  cursor =
+    grn_table_cursor_open(ctx, db, NULL, 0, NULL, 0, 0, -1, GRN_CURSOR_BY_ID);
   if (!cursor) {
     return;
   }
@@ -1060,9 +1105,9 @@ grn_db_reindex(grn_ctx *ctx, grn_obj *db)
     }
 
     switch (object->header.type) {
-    case GRN_TABLE_HASH_KEY :
-    case GRN_TABLE_PAT_KEY :
-    case GRN_TABLE_DAT_KEY :
+    case GRN_TABLE_HASH_KEY:
+    case GRN_TABLE_PAT_KEY:
+    case GRN_TABLE_DAT_KEY:
       grn_obj_reindex(ctx, object);
       break;
     default:
@@ -1083,7 +1128,10 @@ grn_table_reindex(grn_ctx *ctx, grn_obj *table)
 {
   grn_hash *columns;
 
-  columns = grn_hash_create(ctx, NULL, sizeof(grn_id), 0,
+  columns = grn_hash_create(ctx,
+                            NULL,
+                            sizeof(grn_id),
+                            0,
                             GRN_OBJ_TABLE_HASH_KEY | GRN_HASH_TINY);
   if (!columns) {
     ERR(GRN_NO_MEMORY_AVAILABLE,
@@ -1112,8 +1160,7 @@ grn_data_column_reindex(grn_ctx *ctx, grn_obj *data_column)
     grn_token_column_build(ctx, data_column);
   }
 
-  for (hooks = DB_OBJ(data_column)->hooks[GRN_HOOK_SET];
-       hooks;
+  for (hooks = DB_OBJ(data_column)->hooks[GRN_HOOK_SET]; hooks;
        hooks = hooks->next) {
     grn_obj_default_set_value_hook_data *data = (void *)GRN_NEXT_ADDR(hooks);
     grn_obj *target = grn_ctx_at(ctx, data->target);
@@ -1138,22 +1185,22 @@ grn_obj_reindex(grn_ctx *ctx, grn_obj *obj)
   }
 
   switch (obj->header.type) {
-  case GRN_DB :
+  case GRN_DB:
     grn_db_reindex(ctx, obj);
     break;
-  case GRN_TABLE_HASH_KEY :
-  case GRN_TABLE_PAT_KEY :
-  case GRN_TABLE_DAT_KEY :
+  case GRN_TABLE_HASH_KEY:
+  case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
     grn_table_reindex(ctx, obj);
     break;
-  case GRN_COLUMN_FIX_SIZE :
-  case GRN_COLUMN_VAR_SIZE :
+  case GRN_COLUMN_FIX_SIZE:
+  case GRN_COLUMN_VAR_SIZE:
     grn_data_column_reindex(ctx, obj);
     break;
-  case GRN_COLUMN_INDEX :
+  case GRN_COLUMN_INDEX:
     grn_index_column_rebuild(ctx, obj);
     break;
-  default :
+  default:
     {
       grn_obj type_name;
       GRN_TEXT_INIT(&type_name, 0);
@@ -1176,67 +1223,67 @@ const char *
 grn_obj_type_to_string(uint8_t type)
 {
   switch (type) {
-  case GRN_VOID :
+  case GRN_VOID:
     return "void";
-  case GRN_BULK :
+  case GRN_BULK:
     return "bulk";
-  case GRN_PTR :
+  case GRN_PTR:
     return "ptr";
-  case GRN_UVECTOR :
+  case GRN_UVECTOR:
     return "uvector";
-  case GRN_PVECTOR :
+  case GRN_PVECTOR:
     return "pvector";
-  case GRN_VECTOR :
+  case GRN_VECTOR:
     return "vector";
-  case GRN_MSG :
+  case GRN_MSG:
     return "msg";
-  case GRN_QUERY :
+  case GRN_QUERY:
     return "query";
-  case GRN_ACCESSOR :
+  case GRN_ACCESSOR:
     return "accessor";
-  case GRN_SNIP :
+  case GRN_SNIP:
     return "snip";
-  case GRN_PATSNIP :
+  case GRN_PATSNIP:
     return "patsnip";
-  case GRN_STRING :
+  case GRN_STRING:
     return "string";
-  case GRN_CURSOR_TABLE_HASH_KEY :
+  case GRN_CURSOR_TABLE_HASH_KEY:
     return "cursor:table:hash_key";
-  case GRN_CURSOR_TABLE_PAT_KEY :
+  case GRN_CURSOR_TABLE_PAT_KEY:
     return "cursor:table:pat_key";
-  case GRN_CURSOR_TABLE_DAT_KEY :
+  case GRN_CURSOR_TABLE_DAT_KEY:
     return "cursor:table:dat_key";
-  case GRN_CURSOR_TABLE_NO_KEY :
+  case GRN_CURSOR_TABLE_NO_KEY:
     return "cursor:table:no_key";
-  case GRN_CURSOR_COLUMN_INDEX :
+  case GRN_CURSOR_COLUMN_INDEX:
     return "cursor:column:index";
-  case GRN_CURSOR_COLUMN_GEO_INDEX :
+  case GRN_CURSOR_COLUMN_GEO_INDEX:
     return "cursor:column:geo_index";
-  case GRN_CURSOR_CONFIG :
+  case GRN_CURSOR_CONFIG:
     return "cursor:config";
-  case GRN_TYPE :
+  case GRN_TYPE:
     return "type";
-  case GRN_PROC :
+  case GRN_PROC:
     return "proc";
-  case GRN_EXPR :
+  case GRN_EXPR:
     return "expr";
-  case GRN_TABLE_HASH_KEY :
+  case GRN_TABLE_HASH_KEY:
     return "table:hash_key";
-  case GRN_TABLE_PAT_KEY :
+  case GRN_TABLE_PAT_KEY:
     return "table:pat_key";
-  case GRN_TABLE_DAT_KEY :
+  case GRN_TABLE_DAT_KEY:
     return "table:dat_key";
-  case GRN_TABLE_NO_KEY :
+  case GRN_TABLE_NO_KEY:
     return "table:no_key";
-  case GRN_DB :
+  case GRN_DB:
     return "db";
-  case GRN_COLUMN_FIX_SIZE :
+  case GRN_COLUMN_FIX_SIZE:
     return "column:fix_size";
-  case GRN_COLUMN_VAR_SIZE :
+  case GRN_COLUMN_VAR_SIZE:
     return "column:var_size";
-  case GRN_COLUMN_INDEX :
+  case GRN_COLUMN_INDEX:
     return "column:index";
-  default :
+  default:
     return "unknown";
   }
 }
@@ -1269,25 +1316,25 @@ grn_obj_get_io(grn_ctx *ctx, grn_obj *obj)
   }
 
   switch (obj->header.type) {
-  case GRN_TABLE_PAT_KEY :
+  case GRN_TABLE_PAT_KEY:
     io = ((grn_pat *)obj)->io;
     break;
-  case GRN_TABLE_DAT_KEY :
+  case GRN_TABLE_DAT_KEY:
     io = ((grn_dat *)obj)->io;
     break;
-  case GRN_TABLE_HASH_KEY :
+  case GRN_TABLE_HASH_KEY:
     io = ((grn_hash *)obj)->io;
     break;
-  case GRN_TABLE_NO_KEY :
+  case GRN_TABLE_NO_KEY:
     io = ((grn_array *)obj)->io;
     break;
-  case GRN_COLUMN_VAR_SIZE :
+  case GRN_COLUMN_VAR_SIZE:
     io = ((grn_ja *)obj)->io;
     break;
-  case GRN_COLUMN_FIX_SIZE :
+  case GRN_COLUMN_FIX_SIZE:
     io = ((grn_ra *)obj)->io;
     break;
-  case GRN_COLUMN_INDEX :
+  case GRN_COLUMN_INDEX:
     io = ((grn_ii *)obj)->seg;
     break;
   }
@@ -1308,7 +1355,7 @@ grn_obj_get_disk_usage(grn_ctx *ctx, grn_obj *obj)
   }
 
   switch (obj->header.type) {
-  case GRN_DB :
+  case GRN_DB:
     {
       grn_db *db = (grn_db *)obj;
       usage = grn_obj_get_disk_usage(ctx, db->keys);
@@ -1318,13 +1365,13 @@ grn_obj_get_disk_usage(grn_ctx *ctx, grn_obj *obj)
       usage += grn_obj_get_disk_usage(ctx, (grn_obj *)(db->config));
     }
     break;
-  case GRN_TABLE_DAT_KEY :
+  case GRN_TABLE_DAT_KEY:
     usage = grn_dat_get_disk_usage(ctx, (grn_dat *)obj);
     break;
-  case GRN_COLUMN_INDEX :
+  case GRN_COLUMN_INDEX:
     usage = grn_ii_get_disk_usage(ctx, (grn_ii *)obj);
     break;
-  default :
+  default:
     {
       grn_io *io;
       io = grn_obj_get_io(ctx, obj);
@@ -1418,9 +1465,7 @@ grn_obj_clear_option_values(grn_ctx *ctx, grn_obj *obj)
                            ctx->impl->temporary_options,
                            id & ~GRN_OBJ_TMP_OBJECT);
   } else {
-    rc = grn_db_clear_option_values(ctx,
-                                    grn_ctx_db(ctx),
-                                    id);
+    rc = grn_db_clear_option_values(ctx, grn_ctx_db(ctx), id);
   }
 
   GRN_API_RETURN(rc);
@@ -1431,48 +1476,49 @@ grn_obj_to_script_syntax(grn_ctx *ctx, grn_obj *obj, grn_obj *buffer)
 {
   GRN_API_ENTER;
   switch (obj->header.type) {
-  case GRN_VOID :
+  case GRN_VOID:
     GRN_TEXT_PUTS(ctx, buffer, "null");
     break;
-  case GRN_BULK :
+  case GRN_BULK:
     grn_inspect(ctx, buffer, obj);
     break;
-  case GRN_PTR :
+  case GRN_PTR:
     if (GRN_BULK_VSIZE(obj) == 0) {
       GRN_TEXT_PUTS(ctx, buffer, "null");
     } else {
       grn_obj_to_script_syntax(ctx, GRN_PTR_VALUE(obj), buffer);
     }
     break;
-  case GRN_UVECTOR :
-  case GRN_PVECTOR :
-  case GRN_VECTOR :
+  case GRN_UVECTOR:
+  case GRN_PVECTOR:
+  case GRN_VECTOR:
     grn_inspect(ctx, buffer, obj);
     break;
-  case GRN_ACCESSOR :
+  case GRN_ACCESSOR:
     grn_accessor_to_script_syntax(ctx, obj, buffer);
     break;
-  case GRN_TYPE :
-  case GRN_PROC :
+  case GRN_TYPE:
+  case GRN_PROC:
     {
       char name[GRN_TABLE_MAX_KEY_SIZE];
       int name_length = grn_obj_name(ctx, obj, name, sizeof(name));
       GRN_TEXT_PUT(ctx, buffer, name, name_length);
     }
     break;
-  case GRN_EXPR :
+  case GRN_EXPR:
     grn_expr_to_script_syntax(ctx, obj, buffer);
     break;
-  case GRN_TABLE_HASH_KEY :
-  case GRN_TABLE_PAT_KEY :
-  case GRN_TABLE_DAT_KEY :
-  case GRN_TABLE_NO_KEY :
+  case GRN_TABLE_HASH_KEY:
+  case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
+  case GRN_TABLE_NO_KEY:
     if (grn_obj_is_tiny_hash_table(ctx, obj)) {
       GRN_TEXT_PUTC(ctx, buffer, '{');
       uint32_t i = 0;
       grn_obj key;
       GRN_TEXT_INIT(&key, GRN_OBJ_DO_SHALLOW_COPY);
-      GRN_TABLE_EACH_BEGIN_FLAGS(ctx, obj, cursor, id, GRN_CURSOR_BY_ID) {
+      GRN_TABLE_EACH_BEGIN_FLAGS(ctx, obj, cursor, id, GRN_CURSOR_BY_ID)
+      {
         if (i > 0) {
           GRN_TEXT_PUTS(ctx, buffer, ", ");
         }
@@ -1486,7 +1532,8 @@ grn_obj_to_script_syntax(grn_ctx *ctx, grn_obj *obj, grn_obj *buffer)
         grn_obj *value = raw_value;
         grn_obj_to_script_syntax(ctx, value, buffer);
         i++;
-      } GRN_TABLE_EACH_END(ctx, cursor);
+      }
+      GRN_TABLE_EACH_END(ctx, cursor);
       GRN_OBJ_FIN(ctx, &key);
       GRN_TEXT_PUTC(ctx, buffer, '}');
     } else {
@@ -1495,12 +1542,12 @@ grn_obj_to_script_syntax(grn_ctx *ctx, grn_obj *obj, grn_obj *buffer)
       GRN_TEXT_PUT(ctx, buffer, name, name_length);
     }
     break;
-  case GRN_COLUMN_FIX_SIZE :
-  case GRN_COLUMN_VAR_SIZE :
-  case GRN_COLUMN_INDEX :
+  case GRN_COLUMN_FIX_SIZE:
+  case GRN_COLUMN_VAR_SIZE:
+  case GRN_COLUMN_INDEX:
     grn_column_name_(ctx, obj, buffer);
     break;
-  default :
+  default:
     ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
         "[obj][to-script-syntax] unsupported type: %s",
         grn_obj_type_to_string(obj->header.type));
@@ -1509,7 +1556,8 @@ grn_obj_to_script_syntax(grn_ctx *ctx, grn_obj *obj, grn_obj *buffer)
   GRN_API_RETURN(ctx->rc);
 }
 
-static void grn_obj_warm_dispatch(grn_ctx *ctx, grn_obj *obj);
+static void
+grn_obj_warm_dispatch(grn_ctx *ctx, grn_obj *obj);
 
 static void
 grn_db_warm(grn_ctx *ctx, grn_obj *obj)
@@ -1526,7 +1574,8 @@ grn_db_warm(grn_ctx *ctx, grn_obj *obj)
     return;
   }
 
-  GRN_TABLE_EACH_BEGIN_FLAGS(ctx, obj, cursor, id, GRN_CURSOR_BY_ID) {
+  GRN_TABLE_EACH_BEGIN_FLAGS(ctx, obj, cursor, id, GRN_CURSOR_BY_ID)
+  {
     if (id < GRN_N_RESERVED_TYPES) {
       continue;
     }
@@ -1542,7 +1591,8 @@ grn_db_warm(grn_ctx *ctx, grn_obj *obj)
     if (ctx->rc != GRN_SUCCESS) {
       break;
     }
-  } GRN_TABLE_EACH_END(ctx, cursor);
+  }
+  GRN_TABLE_EACH_END(ctx, cursor);
 }
 
 static void
@@ -1553,7 +1603,8 @@ grn_table_warm(grn_ctx *ctx, grn_obj *obj)
     return;
   }
 
-  GRN_HASH_EACH_BEGIN(ctx, columns, cursor, id) {
+  GRN_HASH_EACH_BEGIN(ctx, columns, cursor, id)
+  {
     void *key;
     grn_hash_cursor_get_key(ctx, cursor, &key);
     grn_id column_id = *((grn_id *)key);
@@ -1563,50 +1614,51 @@ grn_table_warm(grn_ctx *ctx, grn_obj *obj)
     if (ctx->rc != GRN_SUCCESS) {
       return;
     }
-  } GRN_HASH_EACH_END(ctx, cursor);
+  }
+  GRN_HASH_EACH_END(ctx, cursor);
 }
 
 static void
 grn_obj_warm_dispatch(grn_ctx *ctx, grn_obj *obj)
 {
   switch (obj->header.type) {
-  case GRN_DB :
+  case GRN_DB:
     grn_db_warm(ctx, obj);
     break;
-  case GRN_TABLE_HASH_KEY :
+  case GRN_TABLE_HASH_KEY:
     if (grn_hash_warm(ctx, (grn_hash *)obj) != GRN_SUCCESS) {
       return;
     }
     grn_table_warm(ctx, obj);
     break;
-  case GRN_TABLE_PAT_KEY :
+  case GRN_TABLE_PAT_KEY:
     if (grn_pat_warm(ctx, (grn_pat *)obj) != GRN_SUCCESS) {
       return;
     }
     grn_table_warm(ctx, obj);
     break;
-  case GRN_TABLE_DAT_KEY :
+  case GRN_TABLE_DAT_KEY:
     if (grn_dat_warm(ctx, (grn_dat *)obj) != GRN_SUCCESS) {
       return;
     }
     grn_table_warm(ctx, obj);
     break;
-  case GRN_TABLE_NO_KEY :
+  case GRN_TABLE_NO_KEY:
     if (grn_array_warm(ctx, (grn_array *)obj) != GRN_SUCCESS) {
       return;
     }
     grn_table_warm(ctx, obj);
     break;
-  case GRN_COLUMN_FIX_SIZE :
+  case GRN_COLUMN_FIX_SIZE:
     grn_ra_warm(ctx, (grn_ra *)obj);
     break;
-  case GRN_COLUMN_VAR_SIZE :
+  case GRN_COLUMN_VAR_SIZE:
     grn_ja_warm(ctx, (grn_ja *)obj);
     break;
-  case GRN_COLUMN_INDEX :
+  case GRN_COLUMN_INDEX:
     grn_ii_warm(ctx, (grn_ii *)obj);
     break;
-  default :
+  default:
     break;
   }
 }
@@ -1642,7 +1694,8 @@ grn_obj_set_error(grn_ctx *ctx,
     ERR(rc,
         "%s[%.*s] %.*s%s%s%s",
         tag,
-        name_size, name,
+        name_size,
+        name,
         (int)GRN_TEXT_LEN(&message),
         GRN_TEXT_VALUE(&message),
         have_path ? ": path:<" : "",
@@ -1652,7 +1705,8 @@ grn_obj_set_error(grn_ctx *ctx,
     ERR(rc,
         "%s[%.*s][%u] %.*s%s%s%s",
         tag,
-        name_size, name,
+        name_size,
+        name,
         id,
         (int)GRN_TEXT_LEN(&message),
         GRN_TEXT_VALUE(&message),
@@ -1692,7 +1746,8 @@ grn_obj_log(grn_ctx *ctx,
             level,
             "%s[%.*s] %.*s%s%s%s",
             tag,
-            name_size, name,
+            name_size,
+            name,
             (int)GRN_TEXT_LEN(&message),
             GRN_TEXT_VALUE(&message),
             have_path ? ": path:<" : "",
@@ -1703,7 +1758,8 @@ grn_obj_log(grn_ctx *ctx,
             level,
             "%s[%.*s][%u] %.*s%s%s%s",
             tag,
-            name_size, name,
+            name_size,
+            name,
             id,
             (int)GRN_TEXT_LEN(&message),
             GRN_TEXT_VALUE(&message),
