@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2017-2018  Brazil
-  Copyright (C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018-2024  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -59,7 +59,8 @@ grn_table_apply_expr(grn_ctx *ctx,
   if (ctx->rc != GRN_SUCCESS) {
     GRN_API_RETURN(ctx->rc);
   }
-  GRN_TABLE_EACH_BEGIN_FLAGS(ctx, table, cursor, id, GRN_CURSOR_BY_ID) {
+  GRN_TABLE_EACH_BEGIN_FLAGS(ctx, table, cursor, id, GRN_CURSOR_BY_ID)
+  {
     grn_obj *value;
     value = grn_expr_executor_exec(ctx, &executor, id);
     if (ctx->rc != GRN_SUCCESS) {
@@ -71,7 +72,8 @@ grn_table_apply_expr(grn_ctx *ctx,
     if (ctx->rc != GRN_SUCCESS) {
       break;
     }
-  } GRN_TABLE_EACH_END(ctx, cursor);
+  }
+  GRN_TABLE_EACH_END(ctx, cursor);
   grn_expr_executor_fin(ctx, &executor);
 
   GRN_API_RETURN(ctx->rc);
@@ -91,21 +93,22 @@ grn_table_find_reference_object(grn_ctx *ctx, grn_obj *table)
 
   table_id = DB_OBJ(table)->id;
 
-  GRN_DB_EACH_SPEC_BEGIN(ctx, cursor, id, spec) {
+  GRN_DB_EACH_SPEC_BEGIN(ctx, cursor, id, spec)
+  {
     if (id == table_id) {
       continue;
     }
 
     switch (spec->header.type) {
-    case GRN_TABLE_HASH_KEY :
-    case GRN_TABLE_PAT_KEY :
-    case GRN_TABLE_DAT_KEY :
+    case GRN_TABLE_HASH_KEY:
+    case GRN_TABLE_PAT_KEY:
+    case GRN_TABLE_DAT_KEY:
       if (spec->header.domain == table_id) {
         reference_object_id = id;
       }
       break;
-    case GRN_COLUMN_VAR_SIZE :
-    case GRN_COLUMN_FIX_SIZE :
+    case GRN_COLUMN_VAR_SIZE:
+    case GRN_COLUMN_FIX_SIZE:
       if (spec->header.domain == table_id) {
         break;
       }
@@ -113,14 +116,15 @@ grn_table_find_reference_object(grn_ctx *ctx, grn_obj *table)
         reference_object_id = id;
       }
       break;
-    default :
+    default:
       break;
     }
 
     if (reference_object_id != GRN_ID_NIL) {
       break;
     }
-  } GRN_DB_EACH_SPEC_END(ctx, cursor);
+  }
+  GRN_DB_EACH_SPEC_END(ctx, cursor);
 
   GRN_API_RETURN(reference_object_id);
 }
@@ -131,7 +135,8 @@ grn_table_copy_array(grn_ctx *ctx, grn_obj *from, grn_obj *to)
   grn_obj missing_ids;
   GRN_RECORD_INIT(&missing_ids, GRN_OBJ_VECTOR, DB_OBJ(from)->id);
   grn_id previous_from_id = GRN_ID_NIL;
-  GRN_TABLE_EACH_BEGIN(ctx, from, cursor, from_id) {
+  GRN_TABLE_EACH_BEGIN(ctx, from, cursor, from_id)
+  {
     for (; previous_from_id < from_id - 1; previous_from_id++) {
       grn_id to_id = grn_table_add(ctx, to, NULL, 0, NULL);
       if (to_id != previous_from_id + 1) {
@@ -142,8 +147,10 @@ grn_table_copy_array(grn_ctx *ctx, grn_obj *from, grn_obj *to)
             "<%u> != <%u>: <%.*s> -> <%.*s>",
             to_id,
             previous_from_id + 1,
-            from_name_size, from_name,
-            to_name_size, to_name);
+            from_name_size,
+            from_name,
+            to_name_size,
+            to_name);
         break;
       }
       GRN_RECORD_PUT(ctx, &missing_ids, to_id);
@@ -156,12 +163,15 @@ grn_table_copy_array(grn_ctx *ctx, grn_obj *from, grn_obj *to)
           "[table][copy] failed to add a record for ID: <%u>: "
           "<%.*s> -> <%.*s>",
           from_id,
-          from_name_size, from_name,
-          to_name_size, to_name);
+          from_name_size,
+          from_name,
+          to_name_size,
+          to_name);
       break;
     }
     previous_from_id = from_id;
-  } GRN_TABLE_EACH_END(ctx, cursor);
+  }
+  GRN_TABLE_EACH_END(ctx, cursor);
 
   if (ctx->rc == GRN_SUCCESS) {
     size_t n_missing_ids = GRN_RECORD_VECTOR_SIZE(&missing_ids);
@@ -178,14 +188,18 @@ grn_table_copy_array(grn_ctx *ctx, grn_obj *from, grn_obj *to)
 static void
 grn_table_copy_same_key_type(grn_ctx *ctx, grn_obj *from, grn_obj *to)
 {
-  GRN_TABLE_EACH_BEGIN_FLAGS(ctx, from, cursor, from_id,
-                             GRN_CURSOR_BY_KEY | GRN_CURSOR_ASCENDING) {
-    void   *key;
-    int     key_size;
-    grn_id  to_id;
+  GRN_TABLE_EACH_BEGIN_FLAGS(ctx,
+                             from,
+                             cursor,
+                             from_id,
+                             GRN_CURSOR_BY_KEY | GRN_CURSOR_ASCENDING)
+  {
+    void *key;
+    int key_size;
+    grn_id to_id;
 
     key_size = grn_table_cursor_get_key(ctx, cursor, &key);
-    to_id    = grn_table_add(ctx, to, key, (unsigned int)key_size, NULL);
+    to_id = grn_table_add(ctx, to, key, (unsigned int)key_size, NULL);
     if (to_id == GRN_ID_NIL) {
       char from_name[GRN_TABLE_MAX_KEY_SIZE];
       int from_name_size;
@@ -209,13 +223,16 @@ grn_table_copy_same_key_type(grn_ctx *ctx, grn_obj *from, grn_obj *to)
           "<%.*s> -> <%.*s>",
           (int)GRN_TEXT_LEN(&inspected_key),
           GRN_TEXT_VALUE(&inspected_key),
-          from_name_size, from_name,
-          to_name_size, to_name);
+          from_name_size,
+          from_name,
+          to_name_size,
+          to_name);
       GRN_OBJ_FIN(ctx, &inspected_key);
       GRN_OBJ_FIN(ctx, &key_buffer);
       break;
     }
-  } GRN_TABLE_EACH_END(ctx, cursor);
+  }
+  GRN_TABLE_EACH_END(ctx, cursor);
 }
 
 static void
@@ -235,8 +252,12 @@ grn_table_copy_different(grn_ctx *ctx, grn_obj *from, grn_obj *to)
     GRN_VALUE_FIX_SIZE_INIT(&to_key_buffer, 0, to->header.domain);
   }
 
-  GRN_TABLE_EACH_BEGIN_FLAGS(ctx, from, cursor, from_id,
-                             GRN_CURSOR_BY_KEY | GRN_CURSOR_ASCENDING) {
+  GRN_TABLE_EACH_BEGIN_FLAGS(ctx,
+                             from,
+                             cursor,
+                             from_id,
+                             GRN_CURSOR_BY_KEY | GRN_CURSOR_ASCENDING)
+  {
     void *key;
     int key_size;
     grn_rc cast_rc;
@@ -271,14 +292,17 @@ grn_table_copy_different(grn_ctx *ctx, grn_obj *from, grn_obj *to)
           GRN_TEXT_VALUE(&inspected_key),
           (int)GRN_TEXT_LEN(&inspected_to_key_type),
           GRN_TEXT_VALUE(&inspected_to_key_type),
-          from_name_size, from_name,
-          to_name_size, to_name);
+          from_name_size,
+          from_name,
+          to_name_size,
+          to_name);
       GRN_OBJ_FIN(ctx, &inspected_key);
       GRN_OBJ_FIN(ctx, &inspected_to_key_type);
       break;
     }
 
-    to_id = grn_table_add(ctx, to,
+    to_id = grn_table_add(ctx,
+                          to,
                           GRN_BULK_HEAD(&to_key_buffer),
                           (unsigned int)GRN_BULK_VSIZE(&to_key_buffer),
                           NULL);
@@ -303,13 +327,16 @@ grn_table_copy_different(grn_ctx *ctx, grn_obj *from, grn_obj *to)
           GRN_TEXT_VALUE(&inspected_from_key),
           (int)GRN_TEXT_LEN(&inspected_to_key),
           GRN_TEXT_VALUE(&inspected_to_key),
-          from_name_size, from_name,
-          to_name_size, to_name);
+          from_name_size,
+          from_name,
+          to_name_size,
+          to_name);
       GRN_OBJ_FIN(ctx, &inspected_from_key);
       GRN_OBJ_FIN(ctx, &inspected_to_key);
       break;
     }
-  } GRN_TABLE_EACH_END(ctx, cursor);
+  }
+  GRN_TABLE_EACH_END(ctx, cursor);
   GRN_OBJ_FIN(ctx, &from_key_buffer);
   GRN_OBJ_FIN(ctx, &to_key_buffer);
 }
@@ -327,7 +354,8 @@ grn_table_copy(grn_ctx *ctx, grn_obj *from, grn_obj *to)
     ERR(GRN_OPERATION_NOT_SUPPORTED,
         "[table][copy] from table and to table is the same: "
         "<%.*s>",
-        from_name_size, from_name);
+        from_name_size,
+        from_name);
     goto exit;
   }
 
@@ -344,8 +372,10 @@ grn_table_copy(grn_ctx *ctx, grn_obj *from, grn_obj *to)
     ERR(GRN_OPERATION_NOT_SUPPORTED,
         "[table][copy] copy from/to TABLE_NO_KEY isn't supported: "
         "<%.*s> -> <%.*s>",
-        from_name_size, from_name,
-        to_name_size, to_name);
+        from_name_size,
+        from_name,
+        to_name_size,
+        to_name);
     goto exit;
   }
 
@@ -357,7 +387,7 @@ grn_table_copy(grn_ctx *ctx, grn_obj *from, grn_obj *to)
     grn_table_copy_different(ctx, from, to);
   }
 
-exit :
+exit:
   GRN_API_RETURN(ctx->rc);
 }
 
@@ -393,11 +423,8 @@ grn_table_get_score(grn_ctx *ctx, grn_obj *table, grn_id id)
 
   while (parent && (parent->header.flags & GRN_OBJ_WITH_SUBREC)) {
     uint32_t parent_value_size;
-    grn_rset_recinfo *parent_ri =
-      (grn_rset_recinfo *)grn_obj_get_value_(ctx,
-                                             parent,
-                                             parent_record_id,
-                                             &parent_value_size);
+    grn_rset_recinfo *parent_ri = (grn_rset_recinfo *)
+      grn_obj_get_value_(ctx, parent, parent_record_id, &parent_value_size);
     if (parent_value_size == 0) {
       break;
     }
@@ -452,7 +479,8 @@ grn_table_get_duplicated_keys(grn_ctx *ctx,
   {
     grn_obj *domain = grn_ctx_at(ctx, table->header.domain);
     keys = grn_table_create(ctx,
-                            NULL, 0,
+                            NULL,
+                            0,
                             NULL,
                             GRN_TABLE_HASH_KEY | GRN_OBJ_KEY_LARGE,
                             domain,
@@ -503,11 +531,13 @@ grn_table_get_duplicated_keys(grn_ctx *ctx,
                              table,
                              cursor,
                              id,
-                             GRN_CURSOR_ASCENDING | GRN_CURSOR_BY_ID) {
+                             GRN_CURSOR_ASCENDING | GRN_CURSOR_BY_ID)
+  {
     void *key;
     int key_size;
     key_size = grn_table_cursor_get_key(ctx, cursor, &key);
-    grn_id keys_id = grn_table_add(ctx, keys, key, (unsigned int)key_size, NULL);
+    grn_id keys_id =
+      grn_table_add(ctx, keys, key, (unsigned int)key_size, NULL);
     if (keys_id == GRN_ID_NIL) {
       char name[GRN_TABLE_MAX_KEY_SIZE];
       int name_size;
@@ -557,7 +587,8 @@ grn_table_get_duplicated_keys(grn_ctx *ctx,
           message);
       break;
     }
-  } GRN_TABLE_EACH_END(ctx, cursor);
+  }
+  GRN_TABLE_EACH_END(ctx, cursor);
   GRN_OBJ_FIN(ctx, &record_buffer);
   if (ctx->rc != GRN_SUCCESS) {
     goto exit;
@@ -565,7 +596,8 @@ grn_table_get_duplicated_keys(grn_ctx *ctx,
 
   grn_obj records_buffer;
   GRN_RECORD_INIT(&records_buffer, GRN_OBJ_VECTOR, DB_OBJ(table)->id);
-  GRN_TABLE_EACH_BEGIN(ctx, keys, cursor, id) {
+  GRN_TABLE_EACH_BEGIN(ctx, keys, cursor, id)
+  {
     GRN_BULK_REWIND(&records_buffer);
     grn_obj_get_value(ctx, records, id, &records_buffer);
     if (ctx->rc != GRN_SUCCESS) {
@@ -586,7 +618,8 @@ grn_table_get_duplicated_keys(grn_ctx *ctx,
     if (GRN_RECORD_VECTOR_SIZE(&records_buffer) == 1) {
       grn_table_cursor_delete(ctx, cursor);
     }
-  } GRN_TABLE_EACH_END(ctx, cursor);
+  }
+  GRN_TABLE_EACH_END(ctx, cursor);
   GRN_OBJ_FIN(ctx, &records_buffer);
   if (ctx->rc != GRN_SUCCESS) {
     goto exit;
@@ -595,7 +628,7 @@ grn_table_get_duplicated_keys(grn_ctx *ctx,
   *duplicated_keys = keys;
   keys = NULL;
 
-exit :
+exit:
   if (keys) {
     grn_obj_close(ctx, keys);
   }
@@ -627,7 +660,8 @@ grn_table_have_duplicated_keys(grn_ctx *ctx, grn_obj *table)
   {
     grn_obj *domain = grn_ctx_at(ctx, table->header.domain);
     keys = grn_table_create(ctx,
-                            NULL, 0,
+                            NULL,
+                            0,
                             NULL,
                             GRN_TABLE_HASH_KEY | GRN_OBJ_KEY_LARGE,
                             domain,
@@ -653,16 +687,14 @@ grn_table_have_duplicated_keys(grn_ctx *ctx, grn_obj *table)
                              table,
                              cursor,
                              id,
-                             GRN_CURSOR_ASCENDING | GRN_CURSOR_BY_ID) {
+                             GRN_CURSOR_ASCENDING | GRN_CURSOR_BY_ID)
+  {
     void *key;
     int key_size;
     key_size = grn_table_cursor_get_key(ctx, cursor, &key);
     int added = 0;
-    grn_id keys_id = grn_table_add(ctx,
-                                   keys,
-                                   key,
-                                   (unsigned int)key_size,
-                                   &added);
+    grn_id keys_id =
+      grn_table_add(ctx, keys, key, (unsigned int)key_size, &added);
     if (keys_id == GRN_ID_NIL) {
       char name[GRN_TABLE_MAX_KEY_SIZE];
       int name_size;
@@ -697,9 +729,10 @@ grn_table_have_duplicated_keys(grn_ctx *ctx, grn_obj *table)
       have_duplicated_keys = true;
       break;
     }
-  } GRN_TABLE_EACH_END(ctx, cursor);
+  }
+  GRN_TABLE_EACH_END(ctx, cursor);
 
-exit :
+exit:
   if (keys) {
     grn_obj_close(ctx, keys);
   }
@@ -721,11 +754,8 @@ grn_table_all_columns(grn_ctx *ctx, grn_obj *table)
   CRITICAL_SECTION_ENTER(ctx->impl->columns_cache_lock);
   void *value;
   grn_hash *columns = NULL;
-  grn_id cache_id = grn_hash_get(ctx,
-                                 ctx->impl->columns_cache,
-                                 &id,
-                                 sizeof(grn_id),
-                                 &value);
+  grn_id cache_id =
+    grn_hash_get(ctx, ctx->impl->columns_cache, &id, sizeof(grn_id), &value);
   if (cache_id != GRN_ID_NIL) {
     columns = *((grn_hash **)value);
   } else {
@@ -741,7 +771,7 @@ grn_table_all_columns(grn_ctx *ctx, grn_obj *table)
                                 NULL,
                                 sizeof(grn_id),
                                 0,
-                                GRN_OBJ_TABLE_HASH_KEY|GRN_HASH_TINY);
+                                GRN_OBJ_TABLE_HASH_KEY | GRN_HASH_TINY);
       grn_table_columns(ctx, table, "", 0, (grn_obj *)columns);
       *((grn_hash **)value) = columns;
     }
@@ -750,3 +780,15 @@ grn_table_all_columns(grn_ctx *ctx, grn_obj *table)
 
   return columns;
 }
+
+bool
+grn_table_have_tokenizer(grn_ctx *ctx, grn_obj *table)
+{
+  if (!GRN_OBJ_TABLEP(table)) {
+    return false;
+  }
+
+  grn_obj *tokenizer = NULL;
+  grn_table_get_info(ctx, table, NULL, NULL, &tokenizer, NULL, NULL);
+  return tokenizer != NULL;
+}
